local AddonName, GGL = ...

local L = GGL.Locale

local EMPTY_TABLE = {}

-- Global
GGL.Global = {}
_G.GoldGoblinLoot = GGL.Global

-- used time
GGL.GetTime = GetServerTime

-- Load time
GGL.LOAD_TIME = GGL.GetTime()

-- db
GoldGoblinLootDB = {}

-- classic era locals
local LE_EXPANSION_CLASSIC = LE_EXPANSION_CLASSIC or 0
local LE_EXPANSION_BURNING_CRUSADE = LE_EXPANSION_BURNING_CRUSADE or 1
local LE_EXPANSION_WRATH_OF_THE_LICH_KING = LE_EXPANSION_WRATH_OF_THE_LICH_KING or 2
local LE_EXPANSION_CATACLYSM = LE_EXPANSION_CATACLYSM or 3
local LE_EXPANSION_MISTS_OF_PANDARIA = LE_EXPANSION_MISTS_OF_PANDARIA or 4
local LE_EXPANSION_WARLORDS_OF_DRAENOR = LE_EXPANSION_WARLORDS_OF_DRAENOR or 5
local LE_EXPANSION_LEGION = LE_EXPANSION_LEGION or 6
local LE_EXPANSION_BATTLE_FOR_AZEROTH = LE_EXPANSION_BATTLE_FOR_AZEROTH or 7
local LE_EXPANSION_SHADOWLANDS = LE_EXPANSION_SHADOWLANDS or 8

local Constants = {}

local function ReverseTable(t)
	local newT = {}
	for i = 1, #t do
		newT[t[i]] = i
	end
	return newT
end

-- Addon
GGL.ADDON_VERSION = "v1.11.1"
if GGL.ADDON_VERSION == "@".."project-version".."@" then
	GGL.ADDON_VERSION = "dev"
end

local WOW_PROJECT_ID = _G.WOW_PROJECT_ID or 99
local WOW_PROJECT_MAINLINE = _G.WOW_PROJECT_MAINLINE or 99
local WOW_PROJECT_CLASSIC = _G.WOW_PROJECT_CLASSIC or 1
local WOW_PROJECT_BURNING_CRUSADE_CLASSIC = _G.WOW_PROJECT_BURNING_CRUSADE_CLASSIC or 2
local WOW_PROJECT_WRATH_CLASSIC = _G.WOW_PROJECT_WRATH_CLASSIC or 11

GGL.IS_CLASSIC = false
GGL.IS_BC = false
GGL.IS_WRATH = false
GGL.IS_RETAIL = false

if WOW_PROJECT_ID == WOW_PROJECT_MAINLINE then
	GGL.IS_RETAIL = true
elseif WOW_PROJECT_ID == WOW_PROJECT_BURNING_CRUSADE_CLASSIC then
	if LE_EXPANSION_LEVEL_CURRENT == LE_EXPANSION_WRATH_OF_THE_LICH_KING then
		GGL.IS_WRATH = true
	elseif LE_EXPANSION_LEVEL_CURRENT == LE_EXPANSION_BURNING_CRUSADE then
		GGL.IS_BCC = true
	end
elseif WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC then
	GGL.IS_WRATH = true
elseif WOW_PROJECT_ID == WOW_PROJECT_CLASSIC then
	GGL.IS_CLASSIC = true
end

GGL.CURRENT_VERSION_NUM = WOW_PROJECT_ID
function GGL:GetGameVersion()
	return WOW_PROJECT_ID
end

-- Account specific
GGL.ACCOUNT_LOCALE = GetLocale()

-- Image path
GGL.PATH = "Interface\\AddOns\\"..AddonName
GGL.MEDIA_PATH = GGL.PATH.."\\media\\"
-- GGL Banner
GGL.BANNER_IMAGE = GGL.MEDIA_PATH.."GGL_Banner"
GGL.BANNER_IMAGE_TEX = {
	["TOP"] = {0, 0.75, 0, 0.2813},
	["BOTTOM"] = {0, 0.75, 0.875, 1},
}

-- delay for addon messages
GGL.SEND_DELAY = 0.25

-- Max error log entrys that get saved after a relog
GGL.MAX_ERROR_LOG_DAYS = 2
GGL.ERROR_LOG_DATE_FORMAT = "%d.%m.%y - %H:%M:%S"

-- Font
GGL.FONT_PATH = GGL.MEDIA_PATH.."PTSansNarrow.ttf"

-- alpha value for lines
GGL.LINE_ALPHA_VALUE = 0.04

-- Other
GGL.PLAYER_NAME_FORMAT_STRING = "%s-%s"

-- max length for names
GGL.MAX_NAME_LENGTH = 25

-- max entrys of top items in challenge TT
GGL.CHALLENGE_TT_MAX_TOP_ITEMS = 5

-- max entry in TopItems
GGL.MAX_TOP_ITEMS = 35

-- Sync Compatibility
GGL.SYNC_VERSION = 28
GGL.DB_VERSION = 2 -- if this increase the options get fully wiped
GGL.DB_PLAYER_REGISTER_VERSION = 1
GGL.DB_SESSION_REGISTER_VERSION = 7

GGL.UNKNOWN_GGL_VERSION = "???"

GGL.ATLAS_TEXTURE = {
	["OK"] = "groupfinder-icon-greencheckmark",
	["X"] = "communities-icon-redx",
	["LOCK"] = "LFG-lock",
	["RELOAD"] = "characterundelete-RestoreButton",
	["WARNING"] = "services-icon-warning",
	["UP"] = "loottoast-arrow-green", -- loottoast-arrow-green
	["UP2"] = "GarrMission_UsefulAbilityIcon",
	["STOP"] = "transmog-icon-remove",
	["+"] = "bags-icon-addslots",
	["config"] = "worldquest-icon-engineering",
}

if GGL.IS_RETAIL then
	GGL.SET_ID_LIST = {
		Set = "s",
		Item = "i",
		PriceSource = "p",
		Zone = "z",
		Instance = "d",
		LootSource = "l",
	}
else
	GGL.SET_ID_LIST = {
		Set = "s",
		Item = "i",
		PriceSource = "p",
		--Zone = "z",
		--Instance = "d",
		LootSource = "l",
	}
end

GGL.SET_ID_LIST_R = {}
for k,v in pairs(GGL.SET_ID_LIST) do GGL.SET_ID_LIST_R[v] = k end

GGL.INVITE_KEY_LIST = {
	NONE = 0,
    INVITED = 1,
    ACCEPTED = 2,
    DECLINED = 3,
}
GGL.INVITE_KEY_LIST_ATLAS_ICONS = {
    [GGL.INVITE_KEY_LIST.INVITED] = "groupfinder-waitdot",
    [GGL.INVITE_KEY_LIST.ACCEPTED] = "groupfinder-icon-greencheckmark",
    [GGL.INVITE_KEY_LIST.DECLINED] = "communities-icon-redx",
}

GGL.ITEM_QUALITY_INFO = {}
for i = 0, 5 do -- Artifacts, Heirloom and WoW Token not needed
	GGL.ITEM_QUALITY_INFO[i] = {
		desc = _G["ITEM_QUALITY"..i.."_DESC"],
		coloredDesc = format("%s%s|r", _G.ITEM_QUALITY_COLORS[i].hex, _G["ITEM_QUALITY"..i.."_DESC"]),
		color = _G.ITEM_QUALITY_COLORS[i]
	}
end

GGL.OPERATORS_LIST = {
	"<",
	">",
	"<=",
	">=",
	"==",
	"~=",
}

GGL.FACTION_ID_TO_NAME = { "Player", "Alliance", "Horde", "Monster" }
GGL.FACTION_NAME_TO_ID = ReverseTable(GGL.FACTION_ID_TO_NAME)

GGL.RACE_ID_REGISTER = {}
GGL.RACE_NAME_TO_ID = {}
for i = 1, 100 do
	local race = C_CreatureInfo.GetRaceInfo(i)
	if race then
		GGL.RACE_ID_REGISTER[race.raceID] = race
		GGL.RACE_NAME_TO_ID[race.raceName] = race.raceID
	end
end

GGL.OPERATORS_LIST_R = {}
for i = 1, #GGL.OPERATORS_LIST do
	GGL.OPERATORS_LIST_R[GGL.OPERATORS_LIST[i]] = i
end
GGL.OPERATORS_DEFAULT = GGL.OPERATORS_LIST_R[">="]

-- class data
GGL.CLASS_ID_TO_NAME = { "WARRIOR", "PALADIN", "HUNTER", "ROGUE", "PRIEST", "DEATHKNIGHT", "SHAMAN", "MAGE", "WARLOCK", "MONK", "DRUID", "DEMONHUNTER", "EVOKER" }
GGL.CLASS_NAME_TO_ID = ReverseTable(GGL.CLASS_ID_TO_NAME)

GGL.LOCALIZED_CLASS_TO_NAME = {}
for k,v in pairs(LOCALIZED_CLASS_NAMES_MALE) do GGL.LOCALIZED_CLASS_TO_NAME[v] = k end
for k,v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do GGL.LOCALIZED_CLASS_TO_NAME[v] = k end

GGL.CLASS_COLORS = RAID_CLASS_COLORS

GGL.FACTION_ICON_ATLAS = {
    ["Horde"] = "communities-create-button-wow-horde",
    ["Alliance"] = "communities-create-button-wow-alliance",
    ["Neutral"] = "AlliedRace-Crest-Nightborne",
}

GGL.BNET_STREAMER_LIST_SORTED = {
	"DECEPTICON#2726",
	--"Nananana#21709"
}
GGL.BNET_STREAMER_LIST = {}
for k,v in ipairs(GGL.BNET_STREAMER_LIST_SORTED) do GGL.BNET_STREAMER_LIST[v] = k end

GGL.GGL_CLUB_IDS = {
    [154393632] = true
}

GGL.INVITE_RESTRICTIONS = {
    "NONE",
    "ONE_WOW_ACCOUNT",
    "ONE_WOW_CHARACTER"
}
for i = 1, #GGL.INVITE_RESTRICTIONS do GGL.INVITE_RESTRICTIONS[GGL.INVITE_RESTRICTIONS[i]] = i end
GGL.INVITE_RESTRICTIONS_DESCRIPTION = {
	[GGL.INVITE_RESTRICTIONS.NONE] 					= L["None"],
	[GGL.INVITE_RESTRICTIONS.ONE_WOW_ACCOUNT] 		= L["Only one Account"],
	[GGL.INVITE_RESTRICTIONS.ONE_WOW_CHARACTER] 	= L["Only one Character"],
}

-- ##################
-- GameVersions
-- ##################
GGL.ALLOWED_PROJECT_ID_INV_SEND = {
    [WOW_PROJECT_MAINLINE] 					= WOW_PROJECT_ID == WOW_PROJECT_MAINLINE,
    [WOW_PROJECT_WRATH_CLASSIC]  			= WOW_PROJECT_ID == WOW_PROJECT_MAINLINE or WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC,
    [WOW_PROJECT_BURNING_CRUSADE_CLASSIC]  	= WOW_PROJECT_ID == WOW_PROJECT_MAINLINE or WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC or WOW_PROJECT_ID == WOW_PROJECT_BURNING_CRUSADE_CLASSIC,
    [WOW_PROJECT_CLASSIC]   				= WOW_PROJECT_ID == WOW_PROJECT_MAINLINE or WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC or WOW_PROJECT_ID == WOW_PROJECT_BURNING_CRUSADE_CLASSIC or WOW_PROJECT_ID == WOW_PROJECT_CLASSIC,
}

-- https://wow.tools/files/#search=interface%2Fglues%2Fcommon%2Fglues-wow-&page=1&sort=0&desc=asc
GGL.PROJECT_ID_DISPLAY_STRINGS = {
    [WOW_PROJECT_MAINLINE]  				= "",
    [WOW_PROJECT_BURNING_CRUSADE_CLASSIC]   = "|cffffd70a[BC]|r", -- 131194 |cffffd70abc|r
    [WOW_PROJECT_WRATH_CLASSIC]       		= "|cffffd70a[W]|r", -- 131194 |cffffd70abc|r
    [WOW_PROJECT_CLASSIC]   				= "|cffffd70a[C]|r", -- 538639
}

-- ##################
-- Flags
-- ##################
GGL.MAIN_SOURCE_FLAG = {
	NONE = 65,		-- none base is 64 and we count it as npc loot
	NPC = 1,
	ITEM = 2,
	OBJECT = 4,
    ZONE = 8,		-- Fishing, mining, etc-
	EXTRA = 16,
}

GGL.MAIN_SOURCE_FLAG_ICONS = {
	[GGL.MAIN_SOURCE_FLAG.NONE] 	= nil, -- hidden
	[GGL.MAIN_SOURCE_FLAG.NPC] 		= nil, -- hidden
	[GGL.MAIN_SOURCE_FLAG.ITEM] 	= nil, -- hidden
	[GGL.MAIN_SOURCE_FLAG.OBJECT] 	= 132594, -- Box icon fallback
	[GGL.MAIN_SOURCE_FLAG.ZONE] 	= nil, -- hidden
	[GGL.MAIN_SOURCE_FLAG.EXTRA] 	= nil, -- hidden
}

GGL.SUB_SOURCE_FLAG = {
    NONE = 0,
    -- Professsions
	DISENCHANTING = 1,
	ENGINEERING = 2,
	MILLING = 4,
    PROSPECTING = 8,

    -- Farm professions
    FISHING = 16,
    HERBALISM = 32,
    MINING = 64,
    SKINNING = 128,

    -- Classes
    PICK_POCKET = 256,

    -- Lootboxes / Chests
    OPENING = 512,

    -- Scrapping
    SCRAPPING = 1024,

	-- Island expedition
	ISLAND_EXPEDITION = 2048,

	-- Quest and Worldquests
	QUEST = 4096,
	WORLD_QUEST = 8192,

	-- Special loot items
	SPECIAL = 16384,

	-- Mission tables
	MISSION_TABLE = 32768,
}

GGL.ZONE_FILTER_IGNORE = {
	[GGL.MAIN_SOURCE_FLAG.ITEM] = {
		[GGL.SUB_SOURCE_FLAG.OPENING] = true,
		[GGL.SUB_SOURCE_FLAG.DISENCHANTING] = true,
		[GGL.SUB_SOURCE_FLAG.MILLING] = true,
		[GGL.SUB_SOURCE_FLAG.PROSPECTING] = true,
		[GGL.SUB_SOURCE_FLAG.SCRAPPING] = true,
		[GGL.SUB_SOURCE_FLAG.SPECIAL] = true,
	},
	[GGL.MAIN_SOURCE_FLAG.ZONE] = {
		[GGL.SUB_SOURCE_FLAG.ISLAND_EXPEDITION] = true,
	},
	[GGL.MAIN_SOURCE_FLAG.EXTRA] = true
}

GGL.SUB_SOURCE_FLAG_ICONS = {
	[GGL.SUB_SOURCE_FLAG.NONE] 				= nil, -- hidden
	[GGL.SUB_SOURCE_FLAG.DISENCHANTING] 	= GetSpellTexture(13262),
	[GGL.SUB_SOURCE_FLAG.ENGINEERING] 		= GetSpellTexture(49383),
	[GGL.SUB_SOURCE_FLAG.MILLING] 			= GetSpellTexture(51005),
	[GGL.SUB_SOURCE_FLAG.PROSPECTING] 		= GetSpellTexture(31252),
	[GGL.SUB_SOURCE_FLAG.FISHING] 			= GetSpellTexture(131476),
	[GGL.SUB_SOURCE_FLAG.HERBALISM] 		= GetSpellTexture(2366),
	[GGL.SUB_SOURCE_FLAG.MINING] 			= GetSpellTexture(2575),
	[GGL.SUB_SOURCE_FLAG.SKINNING] 			= GetSpellTexture(8613),
	[GGL.SUB_SOURCE_FLAG.PICK_POCKET] 		= GetSpellTexture(921),
	[GGL.SUB_SOURCE_FLAG.OPENING] 			= 132594, --GetSpellTexture(3365),
	[GGL.SUB_SOURCE_FLAG.SCRAPPING] 		= GetSpellTexture((C_ScrappingMachineUI and C_ScrappingMachineUI.GetScrapSpellID()) or 265742),
	[GGL.SUB_SOURCE_FLAG.ISLAND_EXPEDITION] = 132319,
	[GGL.SUB_SOURCE_FLAG.QUEST] 			= 136349,
	[GGL.SUB_SOURCE_FLAG.WORLD_QUEST] 		= 609811,
	[GGL.SUB_SOURCE_FLAG.SPECIAL] 			= 134498,
	[GGL.SUB_SOURCE_FLAG.MISSION_TABLE] 	= 236344
}

GGL.CATEGORY_FLAG = {
	LOOT = 1,
	PROFESSION = 2,
	FARM_PROFESSION = 4,
	CLASS = 8,
	CHEST = 16,
	SCRAPPING = 32,
	MONEY = 64,
	INVENTORY_CONTAINER = 128,
	QUEST = 256,
	SPECIAL = 512,
	MISSION_TABLE = 1024
}

GGL.LOOT_SOURCE_TYPES = {
    mobs = {
        id = 1,
		name = L["Mob loot"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.NPC,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.NONE,
    },
    lockbox = {
        id = 2,
		name = L["Lockboxes"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.ITEM,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.PICK_POCKET + GGL.SUB_SOURCE_FLAG.OPENING,
    },
    chest = {
        id = 3,
		name = L["Chests"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.OBJECT,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.PICK_POCKET + GGL.SUB_SOURCE_FLAG.OPENING,
    },
    disenchanting = {
        id = 4,
		name = L["Disenchanting"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.ITEM,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.DISENCHANTING,
	},
	fishing = {
        id = 5,
		name = L["Fishing"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.ZONE,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.FISHING,
	},
	herbalism = {
        id = 6,
		name = L["Herbalism"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.OBJECT + GGL.MAIN_SOURCE_FLAG.NPC,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.HERBALISM,
	},
	mining = {
        id = 7,
		name = L["Mining"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.OBJECT + GGL.MAIN_SOURCE_FLAG.NPC,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.MINING,
	},
	skinning = {
        id = 8,
		name = L["Skinning"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.NPC,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.SKINNING,
	},
    pickpocket = {
        id = 9,
		name = L["Pick Pocket"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.NPC,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.PICK_POCKET,
    },
    scrapping = {
        id = 10,
		name = L["Scrapping"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.ITEM,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.SCRAPPING,
    },
	island_expedition = {
        id = 11,
		name = L["Island Expeditions"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.ZONE + GGL.MAIN_SOURCE_FLAG.ITEM,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.ISLAND_EXPEDITION,
    },
	quest = {
        id = 12,
		name = L["Quest"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.ZONE,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.QUEST,
    },
	world_quest = {
        id = 13,
		name = L["World Quest"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.ZONE,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.WORLD_QUEST,
    },
	special = {
        id = 14,
		name = L["Special"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.ITEM,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.SPECIAL,
    },
	mission_table = {
        id = 15,
		name = L["Mission table"],
        mainSourceFilter = GGL.MAIN_SOURCE_FLAG.EXTRA,
        subSourceFilter = GGL.SUB_SOURCE_FLAG.MISSION_TABLE,
    },
}

GGL.LOOT_SOURCE_IDS = {}
for k,v in pairs(GGL.LOOT_SOURCE_TYPES) do
    v.type = k
    GGL.LOOT_SOURCE_IDS[v.id] = v
end

local MAP_TYPE_FLAGS = {
	NONE = 0,
	ZONE = 1,
	DUNGEON = 2,
	RAID = 4,
	INSTANCE = 6,
	ISLAND_EXPEDITION = 12
}
GGL.MAP_TYPE_FLAGS = MAP_TYPE_FLAGS

-- https://wow.gamepedia.com/DifficultyID
GGL.DIFFICULYID_TO_MAP_TYPE_FLAG = {
	[0] =  MAP_TYPE_FLAGS.DUNGEON,		-- default
	[1] = MAP_TYPE_FLAGS.DUNGEON,		-- Normal	party
	[2] = MAP_TYPE_FLAGS.DUNGEON,		-- Heroic	party	isHeroic
	[3] = MAP_TYPE_FLAGS.RAID,			-- 10 Player	raid	toggleDifficultyID: 5
	[4] = MAP_TYPE_FLAGS.RAID,			-- 25 Player	raid	toggleDifficultyID: 6
	[5] = MAP_TYPE_FLAGS.RAID,			-- 10 Player (Heroic)	raid	isHeroic, toggleDifficultyID: 3
	[6] = MAP_TYPE_FLAGS.RAID,			-- 25 Player (Heroic)	raid	isHeroic, toggleDifficultyID: 4
	[7] = MAP_TYPE_FLAGS.RAID,			-- Looking For Raid	raid	(Legacy LFRs; prior to SoO)
	[8] = MAP_TYPE_FLAGS.DUNGEON,		-- Mythic Keystone	party	isHeroic, isChallengeMode
	[9] = MAP_TYPE_FLAGS.RAID,			-- 40 Player	raid
	[11] = MAP_TYPE_FLAGS.DUNGEON,		-- Heroic Scenario	scenario	isHeroic
	[12] = MAP_TYPE_FLAGS.DUNGEON,		-- Normal Scenario	scenario
	[14] = MAP_TYPE_FLAGS.RAID,			-- Normal	raid
	[15] = MAP_TYPE_FLAGS.RAID,			-- Heroic	raid	displayHeroic
	[16] = MAP_TYPE_FLAGS.RAID,			-- Mythic	raid	isHeroic, displayMythic
	[17] = MAP_TYPE_FLAGS.RAID,			-- Looking For Raid	raid
	[18] = MAP_TYPE_FLAGS.RAID,			-- Event	raid
	[19] = MAP_TYPE_FLAGS.DUNGEON,		-- Event	party
	[20] = MAP_TYPE_FLAGS.DUNGEON,		-- Event Scenario	scenario
	[23] = MAP_TYPE_FLAGS.DUNGEON,		-- Mythic	party	isHeroic, displayMythic
	[24] = MAP_TYPE_FLAGS.DUNGEON,		-- Timewalking	party
	[25] = MAP_TYPE_FLAGS.DUNGEON,		-- World PvP Scenario	scenario
	[29] = MAP_TYPE_FLAGS.DUNGEON,		-- PvEvP Scenario	pvp
	[30] = MAP_TYPE_FLAGS.DUNGEON,		-- Event	scenario
	[32] = MAP_TYPE_FLAGS.DUNGEON,		-- World PvP Scenario	scenario
	[33] = MAP_TYPE_FLAGS.RAID,			-- Timewalking	raid
	[34] = MAP_TYPE_FLAGS.RAID,			-- PvP	pvp
	[38] = MAP_TYPE_FLAGS.DUNGEON,		-- Normal	scenario
	[39] = MAP_TYPE_FLAGS.DUNGEON,		-- Heroic	scenario	displayHeroic
	[40] = MAP_TYPE_FLAGS.DUNGEON,		-- Mythic	scenario	displayMythic
	[45] = MAP_TYPE_FLAGS.DUNGEON,		-- PvP	scenario	displayHeroic
	[147] = MAP_TYPE_FLAGS.DUNGEON,		-- Normal	scenario	(Warfronts)
	[148] = MAP_TYPE_FLAGS.RAID,		-- 20 Player	raid	Classic WoW 20mans (ZG, AQ20)
	[149] = MAP_TYPE_FLAGS.RAID,		-- Heroic	scenario	displayHeroic (Warfronts)
	[150] = MAP_TYPE_FLAGS.DUNGEON,		-- Normal	party
	[151] = MAP_TYPE_FLAGS.RAID,		-- Looking For Raid	raid	(Timewalking)
	[152] = MAP_TYPE_FLAGS.DUNGEON,		-- Visions of N'Zoth	scenario
	[153] = MAP_TYPE_FLAGS.DUNGEON,		-- Teeming Island	scenario	displayHeroic
	[167] = MAP_TYPE_FLAGS.DUNGEON,		-- Torghast	scenario
	[168] = MAP_TYPE_FLAGS.DUNGEON,		-- Path of Ascension: Courage	scenario
	[169] = MAP_TYPE_FLAGS.DUNGEON,		-- Ascension: Loyalty	scenario
	[170] = MAP_TYPE_FLAGS.DUNGEON,		-- Path of Ascension: Wisdom	scenario
	[171] = MAP_TYPE_FLAGS.DUNGEON,		-- Path of Ascension: Humility	scenario
	[172] = MAP_TYPE_FLAGS.RAID,		-- World Boss	none
}

-- ##################
-- Event constants
-- ##################
GGL.EVENT_OPENING_FLAGS = {
	[GGL.MAIN_SOURCE_FLAG.ITEM] = {
		[GGL.SUB_SOURCE_FLAG.OPENING] = true,
		[GGL.SUB_SOURCE_FLAG.DISENCHANTING] = true,
		[GGL.SUB_SOURCE_FLAG.MILLING] = true,
		[GGL.SUB_SOURCE_FLAG.PROSPECTING] = true,
		[GGL.SUB_SOURCE_FLAG.SPECIAL] = true,
	},
}

-- ##################
-- Special items
-- ##################
GGL.LOOT_SPECIAL_CAST_ITEMS = {
	--[spellID] = { [itemID] = true } spellID -> creates ItemID's
	[166550] = { -- Flip Card / Card of Omens
		flags = {113355, GGL.MAIN_SOURCE_FLAG.ITEM, GGL.SUB_SOURCE_FLAG.SPECIAL, GGL.CATEGORY_FLAG.SPECIAL},
		[113340] = true,
		[113341] = true,
		[113342] = true,
		[113343] = true,
		[113344] = true,
		[113345] = true,
		[113346] = true,
		[113347] = true,
		[113348] = true,
		[113349] = true,
		[113350] = true,
		[113351] = true,
		[113352] = true,
		[113353] = true,
		[113354] = true,
	},
	[84193] = { -- Deck of Trades / Mysterious Fortune Card
		flags = {60838, GGL.MAIN_SOURCE_FLAG.ITEM, GGL.SUB_SOURCE_FLAG.SPECIAL, GGL.CATEGORY_FLAG.SPECIAL},
		[60844] = true,
		[62605] = true,
		[62602] = true,
		[62557] = true,
		[62571] = true,
		[60845] = true,
		[60840] = true,
		[62561] = true,
		[62587] = true,
		[62606] = true,
		[62591] = true,
		[62570] = true,
		[62581] = true,
		[62598] = true,
		[62573] = true,
		[62552] = true,
		[62553] = true,
		[62603] = true,
		[62576] = true,
		[62566] = true,
		[62589] = true,
		[60843] = true,
		[62559] = true,
		[62582] = true,
		[62247] = true,
		[62583] = true,
		[62579] = true,
		[62584] = true,
		[62574] = true,
		[60839] = true,
		[62575] = true,
		[62554] = true,
		[62565] = true,
		[60841] = true,
		[62577] = true,
		[62556] = true,
		[62588] = true,
		[62567] = true,
		[60842] = true,
		[62578] = true,
		[62568] = true,
		[62558] = true,
		[62590] = true,
		[62569] = true,
		[62580] = true,
		[62560] = true,
		[62572] = true,
		[62562] = true,
		[62585] = true,
		[62604] = true,
		[62601] = true,
		[62564] = true,
		[62586] = true,
		[62555] = true,
		[62246] = true,
		[62599] = true,
		[62600] = true,
		[62563] = true,
	},
}

GGL.ALWAYS_USE_VENDOR_PRICE_FOR_ITEMID = {}
for spellID, spellData in pairs(GGL.LOOT_SPECIAL_CAST_ITEMS) do
	for itemID in pairs(spellData) do
		if itemID ~= "flags" then
			GGL.ALWAYS_USE_VENDOR_PRICE_FOR_ITEMID[itemID] = true
		end
	end
end

-- ##################
-- MapData
-- ##################
local MAP_XPACK_FLAG = {
	CLASSIC = 1,
	BC = 2,
	WOTLK = 4,
	CATA = 8,
	MOP = 16,
	WOD = 32,
	LEGION = 64,
	BFA = 128,
}
GGL.MAP_XPACK_FLAG = MAP_XPACK_FLAG

GGL.CONTINENT_MAP_LIST = {
	list={12,13,101,113,424,572,619,875,876,824,905,948,1550,407,1409,1978},
	[12] = {12,1,7,10,57,62,63,64,65,66,69,70,71,76,77,78,80,81,83,85,88,89,97,103,106,198,199,249,460,461,462,463,468,1339,1527}, -- Kalimdor
	[13] = {13,14,15,17,18,21,22,23,25,26,27,32,36,37,42,47,48,49,50,51,52,56,84,87,90,94,95,110,122,201,202,203,204,205,241,210,217,224,244,245,425,465,467,469,998,1577}, -- Eastern Kingdoms
	[101] = {101,100,102,104,105,107,108,109,111}, -- Outland
	[113] = {113,125,114,115,116,117,118,119,120,121,123,127,170}, -- Northrend
	[424] = {424,371,376,379,388,390,418,422,433,504,507,554,1530}, -- Pandaria
	[572] = {572,525,534,535,539,542,543,550,588}, -- Draenor
	[619] = {619,626,630,634,641,646,650,680,739,790}, -- Broken Isles
	[875] = {875,862,863,864,1041,1165,1355}, -- Zandalar
	[876] = {876,895,896,942,1039,1161,1169,1462}, -- Kul Tiras
	[905] = {905,830,882,885}, -- Argus
	[948] = {948,174,194,207,276}, -- The Maelstrom
	[1550] = {1550,1670,1525,1533,1536,1543,1565,1960,1961,1970}, -- The Shadowlands
	[1978] = {1978,2022,2023,2024,2025,2107,2112}, -- Dragon Isles
	-- Special
	[824]={981,1032,1033,1034,1035,1036,1037,1336,1337,1501,1502}, -- Island Espeditions
	[407]={407},  -- Darkmoon Island
	[1409]={1409,1609},  -- Exile's Reach
}

GGL.EXPANSION_NAME = {
	[LE_EXPANSION_CLASSIC] 					= EXPANSION_NAME0,
	[LE_EXPANSION_BURNING_CRUSADE] 			= EXPANSION_NAME1,
	[LE_EXPANSION_WRATH_OF_THE_LICH_KING] 	= EXPANSION_NAME2,
	[LE_EXPANSION_CATACLYSM] 				= EXPANSION_NAME3,
	[LE_EXPANSION_MISTS_OF_PANDARIA] 		= EXPANSION_NAME4,
	[LE_EXPANSION_WARLORDS_OF_DRAENOR] 		= EXPANSION_NAME5,
	[LE_EXPANSION_LEGION] 					= EXPANSION_NAME6,
	[LE_EXPANSION_BATTLE_FOR_AZEROTH] 		= EXPANSION_NAME7,
	[LE_EXPANSION_SHADOWLANDS] 				= EXPANSION_NAME8,
	[LE_EXPANSION_DRAGONFLIGHT or 9] 		= EXPANSION_NAME9,
}

GGL.INSTANCE_MAP_LIST = {
	[LE_EXPANSION_CLASSIC] = { -- classic
		dungeon={227,228,"63:1",230,231,229,232,226,233,234,"311:1","316:1","246:1","64:1",236,238,237,239,240,241},
		raid={741,742,743,744},
	},
	[LE_EXPANSION_BURNING_CRUSADE] = { -- bc
		dungeon={247,248,249,250,251,252,253,254,255,256,257,258,259,260,261,262},
		raid={745,746,747,748,749,750,751,752},
	},
	[LE_EXPANSION_WRATH_OF_THE_LICH_KING] = { -- WotlK
		dungeon={271,272,273,274,275,276,277,278,279,280,281,282,283,284,285,286},
		raid={753,754,755,756,759,757,760,758,761},
	},
	[LE_EXPANSION_CATACLYSM] = { -- Cata
		dungeon={"63:2","64:2",66,184,70,72,69,67,68,65,185,77,76},
		raid={75,73,186,74,78,187},
	},
	[LE_EXPANSION_MISTS_OF_PANDARIA] = { -- MoP
		dungeon={"246:2","311:2","316:2",303,321,312,324,302,313},
		raid={317,330,320,362,369},
	},
	[LE_EXPANSION_WARLORDS_OF_DRAENOR] = { -- WoD
		dungeon={547,385,536,558,537,476,556,559},
		raid={477,457,669},
	},
	[LE_EXPANSION_LEGION] = { -- Legion
		dungeon={777,740,900,800,762,716,721,727,767,726,707},
		raid={768,861,786,875},
	},
	[LE_EXPANSION_BATTLE_FOR_AZEROTH] = { -- BfA
		dungeon={968,1001,1178,1036,1023,1030,1012,1022,1002,1021},
		raid={1031,1176,1177,1179,1180},
	},
	[LE_EXPANSION_SHADOWLANDS] = { -- SL
		dungeon={1182,1183,1184,1185,1186,1187,1188,1189},
		raid={1190,1195},
	},
	[LE_EXPANSION_DRAGONFLIGHT or 9] = { -- DF
		dungeon={1196,1197,1198,1199,1201,1202,1203,1204},
		raid={1206},
	},
}

-- worldMapID, continentMapID, zoneMapID, mapTypeFlag, xPackFlag
GGL.MAP_ID_DATA = {
	-- Main
	[12]={12,12,12,MAP_TYPE_FLAGS.ZONE}, -- Kalimdor
	[13]={13,13,13,MAP_TYPE_FLAGS.ZONE}, -- Eastern Kingdoms
	[101]={101,101,101,MAP_TYPE_FLAGS.ZONE}, -- Outland
	[113]={113,113,113,MAP_TYPE_FLAGS.ZONE}, -- Northrend
	[424]={424,424,424,MAP_TYPE_FLAGS.ZONE}, -- Pandaria
	[572]={572,572,572,MAP_TYPE_FLAGS.ZONE}, -- Draenor
	[619]={619,619,619,MAP_TYPE_FLAGS.ZONE}, -- Broken Isles
	[875]={875,875,875,MAP_TYPE_FLAGS.ZONE}, -- Zandalar
	[876]={876,876,876,MAP_TYPE_FLAGS.ZONE}, -- Kul Tiras
	[905]={905,905,905,MAP_TYPE_FLAGS.ZONE}, -- Argus
	[948]={948,948,948,MAP_TYPE_FLAGS.ZONE}, -- The Maelstrom
	[1550]={1550,1550,1550,MAP_TYPE_FLAGS.ZONE}, -- The Shadowlands
	-- Kalimdor
	[1]={947,12,1,MAP_TYPE_FLAGS.ZONE}, -- Durotar
	[7]={947,12,7,MAP_TYPE_FLAGS.ZONE}, -- Mulgore
	[10]={947,12,10,MAP_TYPE_FLAGS.ZONE}, -- Northern Barrens
	[57]={947,12,57,MAP_TYPE_FLAGS.ZONE}, -- Teldrassil
	[62]={947,12,62,MAP_TYPE_FLAGS.ZONE}, -- Darkshore
	[63]={947,12,63,MAP_TYPE_FLAGS.ZONE}, -- Ashenvale
	[64]={947,12,64,MAP_TYPE_FLAGS.ZONE}, -- Thousand Needles
	[65]={947,12,65,MAP_TYPE_FLAGS.ZONE}, -- Stonetalon Mountains
	[66]={947,12,66,MAP_TYPE_FLAGS.ZONE}, -- Desolace
	[69]={947,12,69,MAP_TYPE_FLAGS.ZONE}, -- Feralas
	[70]={947,12,70,MAP_TYPE_FLAGS.ZONE}, -- Dustwallow Marsh
	[71]={947,12,71,MAP_TYPE_FLAGS.ZONE}, -- Tanaris
	[76]={947,12,76,MAP_TYPE_FLAGS.ZONE}, -- Azshara
	[77]={947,12,77,MAP_TYPE_FLAGS.ZONE}, -- Felwood
	[78]={947,12,78,MAP_TYPE_FLAGS.ZONE}, -- Un'Goro Crater
	[80]={947,12,80,MAP_TYPE_FLAGS.ZONE}, -- Moonglade
	[81]={947,12,81,MAP_TYPE_FLAGS.ZONE}, -- Silithus
	[83]={947,12,83,MAP_TYPE_FLAGS.ZONE}, -- Winterspring
	[85]={947,12,85,MAP_TYPE_FLAGS.ZONE}, -- Orgrimmar
	[88]={947,12,88,MAP_TYPE_FLAGS.ZONE}, -- Thunder Bluff
	[89]={947,12,89,MAP_TYPE_FLAGS.ZONE}, -- Darnassus
	[97]={947,12,97,MAP_TYPE_FLAGS.ZONE}, -- Azuremyst Isle
	[103]={947,12,103,MAP_TYPE_FLAGS.ZONE}, -- The Exodar
	[106]={947,12,106,MAP_TYPE_FLAGS.ZONE}, -- Bloodmyst Isle
	[198]={947,12,198,MAP_TYPE_FLAGS.ZONE}, -- Mount Hyjal
	[199]={947,12,199,MAP_TYPE_FLAGS.ZONE}, -- Southern Barrens
	[249]={947,12,249,MAP_TYPE_FLAGS.ZONE}, -- Uldum
	[460]={947,12,460,MAP_TYPE_FLAGS.ZONE}, -- Shadowglen
	[461]={947,12,461,MAP_TYPE_FLAGS.ZONE}, -- Valley of Trials
	[462]={947,12,462,MAP_TYPE_FLAGS.ZONE}, -- Camp Narache
	[463]={947,12,463,MAP_TYPE_FLAGS.ZONE}, -- Echo Isles
	[468]={947,12,468,MAP_TYPE_FLAGS.ZONE}, -- Ammen Vale
	[1339]={947,12,1339,MAP_TYPE_FLAGS.ZONE}, -- Warsong Gulch
	[1527]={947,12,1527,MAP_TYPE_FLAGS.ZONE}, -- Uldum
	-- Eastern Kingdoms
	[14]={947,13,14,MAP_TYPE_FLAGS.ZONE}, -- Arathi Highlands
	[15]={947,13,15,MAP_TYPE_FLAGS.ZONE}, -- Badlands
	[17]={947,13,17,MAP_TYPE_FLAGS.ZONE}, -- Blasted Lands
	[18]={947,13,18,MAP_TYPE_FLAGS.ZONE}, -- Tirisfal Glades
	[21]={947,13,21,MAP_TYPE_FLAGS.ZONE}, -- Silverpine Forest
	[22]={947,13,22,MAP_TYPE_FLAGS.ZONE}, -- Western Plaguelands
	[23]={947,13,23,MAP_TYPE_FLAGS.ZONE}, -- Eastern Plaguelands
	[25]={947,13,25,MAP_TYPE_FLAGS.ZONE}, -- Hillsbrad Foothills
	[26]={947,13,26,MAP_TYPE_FLAGS.ZONE}, -- The Hinterlands
	[27]={947,13,27,MAP_TYPE_FLAGS.ZONE}, -- Dun Morogh
	[32]={947,13,32,MAP_TYPE_FLAGS.ZONE}, -- Searing Gorge
	[36]={947,13,36,MAP_TYPE_FLAGS.ZONE}, -- Burning Steppes
	[37]={947,13,37,MAP_TYPE_FLAGS.ZONE}, -- Elwynn Forest
	[42]={947,13,42,MAP_TYPE_FLAGS.ZONE}, -- Deadwind Pass
	[47]={947,13,47,MAP_TYPE_FLAGS.ZONE}, -- Duskwood
	[48]={947,13,48,MAP_TYPE_FLAGS.ZONE}, -- Loch Modan
	[49]={947,13,49,MAP_TYPE_FLAGS.ZONE}, -- Redridge Mountains
	[50]={947,13,50,MAP_TYPE_FLAGS.ZONE}, -- Northern Stranglethorn
	[51]={947,13,51,MAP_TYPE_FLAGS.ZONE}, -- Swamp of Sorrows
	[52]={947,13,52,MAP_TYPE_FLAGS.ZONE}, -- Westfall
	[56]={947,13,56,MAP_TYPE_FLAGS.ZONE}, -- Wetlands
	[84]={947,13,84,MAP_TYPE_FLAGS.ZONE}, -- Stormwind City
	[87]={947,13,87,MAP_TYPE_FLAGS.ZONE}, -- Ironforge
	[90]={947,13,90,MAP_TYPE_FLAGS.ZONE}, -- Undercity
	[94]={947,13,94,MAP_TYPE_FLAGS.ZONE}, -- Eversong Woods
	[95]={947,13,95,MAP_TYPE_FLAGS.ZONE}, -- Ghostlands
	[110]={947,13,110,MAP_TYPE_FLAGS.ZONE}, -- Silvermoon City
	[122]={947,13,122,MAP_TYPE_FLAGS.ZONE}, -- Isle of Quel'Danas
	[201]={947,13,201,MAP_TYPE_FLAGS.ZONE}, -- Kelp'thar Forest
	[202]={947,13,202,MAP_TYPE_FLAGS.ZONE}, -- Gilneas City
	[203]={947,13,203,MAP_TYPE_FLAGS.ZONE}, -- Vashj'ir
	[204]={947,13,204,MAP_TYPE_FLAGS.ZONE}, -- Abyssal Depths
	[205]={947,13,205,MAP_TYPE_FLAGS.ZONE}, -- Shimmering Expanse
	[241]={947,13,241,MAP_TYPE_FLAGS.ZONE}, -- Twilight Highlands
	[210]={947,13,210,MAP_TYPE_FLAGS.ZONE}, -- The Cape of Stranglethorn
	[217]={947,13,217,MAP_TYPE_FLAGS.ZONE}, -- Ruins of Gilneas
	[224]={947,13,224,MAP_TYPE_FLAGS.ZONE}, -- Stranglethorn Vale
	[244]={947,13,244,MAP_TYPE_FLAGS.ZONE}, -- Tol Barad
	[245]={947,13,245,MAP_TYPE_FLAGS.ZONE}, -- Tol Barad Peninsula
	[425]={947,13,425,MAP_TYPE_FLAGS.ZONE}, -- Northshire
	[465]={947,13,465,MAP_TYPE_FLAGS.ZONE}, -- Deathknell
	[467]={947,13,467,MAP_TYPE_FLAGS.ZONE}, -- Sunstrider Isle
	[469]={947,13,469,MAP_TYPE_FLAGS.ZONE}, -- New Tinkertown
	[998]={947,13,998,MAP_TYPE_FLAGS.ZONE}, -- Undercity
	[1577]={947,13,1577,MAP_TYPE_FLAGS.ZONE}, -- Gilneas City
	[2070]={947,13,2070,MAP_TYPE_FLAGS.ZONE}, -- Tirisfal Glades
	-- Outland
	[100]={0,101,100,MAP_TYPE_FLAGS.ZONE}, -- Hellfire Peninsula
	[102]={0,101,102,MAP_TYPE_FLAGS.ZONE}, -- Zangarmarsh
	[104]={0,101,104,MAP_TYPE_FLAGS.ZONE}, -- Shadowmoon Valley
	[105]={0,101,105,MAP_TYPE_FLAGS.ZONE}, -- Blade's Edge Mountains
	[107]={0,101,107,MAP_TYPE_FLAGS.ZONE}, -- Nagrand
	[108]={0,101,108,MAP_TYPE_FLAGS.ZONE}, -- Terokkar Forest
	[109]={0,101,109,MAP_TYPE_FLAGS.ZONE}, -- Netherstorm
	[111]={0,101,111,MAP_TYPE_FLAGS.ZONE}, -- Shattrath City
	-- Northrend
	[114]={947,113,114,MAP_TYPE_FLAGS.ZONE}, -- Borean Tundra
	[115]={947,113,115,MAP_TYPE_FLAGS.ZONE}, -- Dragonblight
	[116]={947,113,116,MAP_TYPE_FLAGS.ZONE}, -- Grizzly Hills
	[117]={947,113,117,MAP_TYPE_FLAGS.ZONE}, -- Howling Fjord
	[118]={947,113,118,MAP_TYPE_FLAGS.ZONE}, -- Icecrown
	[119]={947,113,119,MAP_TYPE_FLAGS.ZONE}, -- Sholazar Basin
	[120]={947,113,120,MAP_TYPE_FLAGS.ZONE}, -- The Storm Peaks
	[121]={947,113,121,MAP_TYPE_FLAGS.ZONE}, -- Zul'Drak
	[123]={947,113,123,MAP_TYPE_FLAGS.ZONE}, -- Wintergrasp
	[125]={947,113,125,MAP_TYPE_FLAGS.ZONE}, -- Dalaran
	[127]={947,113,127,MAP_TYPE_FLAGS.ZONE}, -- Crystalsong Forest
	[170]={947,113,170,MAP_TYPE_FLAGS.ZONE}, -- Hrothgar's Landing
	-- Pandaria
	[371]={947,424,371,MAP_TYPE_FLAGS.ZONE}, -- The Jade Forest
	[376]={947,424,376,MAP_TYPE_FLAGS.ZONE}, -- Valley of the Four Winds
	[379]={947,424,379,MAP_TYPE_FLAGS.ZONE}, -- Kun-Lai Summit
	[388]={947,424,388,MAP_TYPE_FLAGS.ZONE}, -- Townlong Steppes
	[390]={947,424,390,MAP_TYPE_FLAGS.ZONE}, -- Vale of Eternal Blossoms
	[418]={947,424,418,MAP_TYPE_FLAGS.ZONE}, -- Krasarang Wilds
	[422]={947,424,422,MAP_TYPE_FLAGS.ZONE}, -- Dread Wastes
	[433]={947,424,433,MAP_TYPE_FLAGS.ZONE}, -- The Veiled Stair
	[504]={947,424,504,MAP_TYPE_FLAGS.ZONE}, -- Isle of Thunder
	[507]={947,424,507,MAP_TYPE_FLAGS.ZONE}, -- Isle of Giants
	[554]={947,424,554,MAP_TYPE_FLAGS.ZONE}, -- Timeless Isle
	[1530]={947,424,1530,MAP_TYPE_FLAGS.ZONE}, -- Vale of Eternal Blossoms
	-- Draenor
	[525]={0,572,525,MAP_TYPE_FLAGS.ZONE}, -- Frostfire Ridge
	[534]={0,572,534,MAP_TYPE_FLAGS.ZONE}, -- Tanaan Jungle
	[535]={0,572,535,MAP_TYPE_FLAGS.ZONE}, -- Talador
	[539]={0,572,539,MAP_TYPE_FLAGS.ZONE}, -- Shadowmoon Valley
	[542]={0,572,542,MAP_TYPE_FLAGS.ZONE}, -- Spires of Arak
	[543]={0,572,543,MAP_TYPE_FLAGS.ZONE}, -- Gorgrond
	[550]={0,572,550,MAP_TYPE_FLAGS.ZONE}, -- Nagrand
	[588]={0,572,588,MAP_TYPE_FLAGS.ZONE}, -- Ashran
	-- Broken Isles
	[630]={947,619,630,MAP_TYPE_FLAGS.ZONE}, -- Azsuna
	[634]={947,619,634,MAP_TYPE_FLAGS.ZONE}, -- Stormheim
	[641]={947,619,641,MAP_TYPE_FLAGS.ZONE}, -- Val'sharah
	[646]={947,619,646,MAP_TYPE_FLAGS.ZONE}, -- Broken Shore
	[650]={947,619,650,MAP_TYPE_FLAGS.ZONE}, -- Highmountain
	[680]={947,619,680,MAP_TYPE_FLAGS.ZONE}, -- Suramar
	[739]={947,619,739,MAP_TYPE_FLAGS.ZONE}, -- Trueshot Lodge
	[790]={947,619,790,MAP_TYPE_FLAGS.ZONE}, -- Eye of Azshara
	[626]={947,619,626,MAP_TYPE_FLAGS.ZONE}, -- Dalaran
	-- Zandalar
	[862]={947,875,862,MAP_TYPE_FLAGS.ZONE}, -- Zuldazar
	[863]={947,875,863,MAP_TYPE_FLAGS.ZONE}, -- Nazmir
	[864]={947,875,864,MAP_TYPE_FLAGS.ZONE}, -- Vol'dun
	[1041]={947,875,1041,MAP_TYPE_FLAGS.ZONE}, -- The Underrot
	[1165]={947,875,1165,MAP_TYPE_FLAGS.ZONE}, -- Dazar'alor
	[1355]={947,875,1355,MAP_TYPE_FLAGS.ZONE}, -- Nazjatar
	-- Kul Tiras
	[895]={947,876,895,MAP_TYPE_FLAGS.ZONE}, -- Tiragarde Sound
	[896]={947,876,896,MAP_TYPE_FLAGS.ZONE}, -- Drustvar
	[942]={947,876,942,MAP_TYPE_FLAGS.ZONE}, -- Stormsong Valley
	[1039]={947,876,1039,MAP_TYPE_FLAGS.ZONE}, -- Shrine of the Storm
	[1161]={947,876,1161,MAP_TYPE_FLAGS.ZONE}, -- Boralus
	[1169]={947,876,1169,MAP_TYPE_FLAGS.ZONE}, -- Tol Dagor
	[1462]={947,876,1462,MAP_TYPE_FLAGS.ZONE}, -- Mechagon Island
	-- Argus
	[830]={947,905,830,MAP_TYPE_FLAGS.ZONE}, -- Krokuun
	[882]={947,905,882,MAP_TYPE_FLAGS.ZONE}, -- Eredath
	[885]={947,905,885,MAP_TYPE_FLAGS.ZONE}, -- Antoran Wastes
	-- The Maelstrom
	[174]={947,948,174,MAP_TYPE_FLAGS.ZONE}, -- The Lost Isles
	[194]={947,948,194,MAP_TYPE_FLAGS.ZONE}, -- Kezan
	[207]={947,948,207,MAP_TYPE_FLAGS.ZONE}, -- Deepholm
	[276]={947,948,276,MAP_TYPE_FLAGS.ZONE}, -- The Maelstrom
	-- The Shadowlands
	[1525]={0,1550,1525,MAP_TYPE_FLAGS.ZONE}, -- Revendreth
	[1533]={0,1550,1533,MAP_TYPE_FLAGS.ZONE}, -- Bastion
	[1536]={0,1550,1536,MAP_TYPE_FLAGS.ZONE}, -- Maldraxxus
	[1543]={0,1550,1543,MAP_TYPE_FLAGS.ZONE}, -- The Maw
	[1565]={0,1550,1565,MAP_TYPE_FLAGS.ZONE}, -- Ardenweald
	[1960]={0,1550,1960,MAP_TYPE_FLAGS.ZONE}, -- The Maw
	[1961]={0,1550,1961,MAP_TYPE_FLAGS.ZONE}, -- Korthia
	[1970]={0,1550,1970,MAP_TYPE_FLAGS.ZONE}, -- Zereth Mortis
	[1670]={0,1550,1670,MAP_TYPE_FLAGS.ZONE}, -- Oribos
	-- Dragon Isles
	[2022]={947,1978,2022,MAP_TYPE_FLAGS.ZONE}, -- The Waking Shores
	[2023]={947,1978,2023,MAP_TYPE_FLAGS.ZONE}, -- Ohn'ahran Plains
	[2024]={947,1978,2024,MAP_TYPE_FLAGS.ZONE}, -- The Azure Span
	[2025]={947,1978,2025,MAP_TYPE_FLAGS.ZONE}, -- Thaldraszus
	[2107]={947,1978,2107,MAP_TYPE_FLAGS.ZONE}, -- The Forbidden Reach
	[2112]={947,1978,2112,MAP_TYPE_FLAGS.ZONE}, -- Valdrakken

	-- ## Island expeditions
	[981]={0,824,981,MAP_TYPE_FLAGS.ISLAND_EXPEDITION},   -- Un'gol Ruins
	[1032]={0,824,1032,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- Skittering Hollow
	[1033]={0,824,1033,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- The Rotting Mire
	[1034]={0,824,1034,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- Verdant Wilds
	[1035]={0,824,1035,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- Molten Cay
	[1036]={0,824,1036,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- The Dread Chain
	[1037]={0,824,1037,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- Whispering Reef
	[1336]={0,824,1336,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- Havenswood
	[1337]={0,824,1337,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- Jorundall
	[1501]={0,824,1501,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- Crestfall
	[1502]={0,824,1502,MAP_TYPE_FLAGS.ISLAND_EXPEDITION}, -- Snowblossom Village

	-- ## Darkmoon Fair
	[407]={0,407,407,MAP_TYPE_FLAGS.ZONE}, -- Darkmoon Island

	-- ## Exile's Reach
	[1409]={0,1409,407,MAP_TYPE_FLAGS.ZONE}, -- Darkmoon Island
	[1609]={0,1409,1609,MAP_TYPE_FLAGS.ZONE}, -- Darkmaul Citadel

}

GGL.INSTANCE_ID_DATA = {
	-- Dummys
	[63]={nil,291}, --Deadmines(Normal)
	[64]={nil,310}, --Shadowfang Keep(Normal)
	[246]={nil,476}, --Scholomance(Normal)
	[311]={nil,431}, --Scarlet Halls(Normal)
	[316]={nil,302}, --Scarlet Monastery(Normal)

	-- Classic // Dungeon
	["63:1"]={0,291}, --Deadmines(Normal)
	["64:1"]={0,310}, --Shadowfang Keep(Normal)
	[226]={0,213}, --Ragefire Chasm
	[227]={0,221}, --Blackfathom Deeps
	[228]={0,242}, --Blackrock Depths
	[229]={0,250}, --Lower Blackrock Spire
	[230]={0,235}, --Dire Maul
	[231]={0,226}, --Gnomeregan
	[232]={0,280}, --Maraudon
	[233]={0,300}, --Razorfen Downs
	[234]={0,301}, --Razorfen Kraul
	[236]={0,317}, --Stratholme
	[237]={0,220}, --The Temple of Atal'hakkar
	[238]={0,225}, --The Stockade
	[239]={0,230}, --Uldaman
	[240]={0,279}, --Wailing Caverns
	[241]={0,219}, --Zul'Farrak
	["246:1"]={0,476}, --Scholomance(Normal)
	["311:1"]={0,431}, --Scarlet Halls(Normal)
	["316:1"]={0,302}, --Scarlet Monastery(Normal)
	[741]={0,232}, --Molten Core
	-- Classic // Raid
	[742]={0,287}, --Blackwing Lair
	[743]={0,247}, --Ruins of Ahn'Qiraj
	[744]={0,319}, --Temple of Ahn'Qiraj

	-- BC // Dungeon
	[247]={1,256}, --Auchenai Crypts
	[248]={1,347}, --Hellfire Ramparts
	[249]={1,348}, --Magisters' Terrace
	[250]={1,272}, --Mana-Tombs
	[251]={1,274}, --Old Hillsbrad Foothills
	[252]={1,258}, --Sethekk Halls
	[253]={1,260}, --Shadow Labyrinth
	[254]={1,269}, --The Arcatraz
	[255]={1,273}, --The Black Morass
	[256]={1,261}, --The Blood Furnace
	[257]={1,266}, --The Botanica
	[258]={1,267}, --The Mechanar
	[259]={1,246}, --The Shattered Halls
	[260]={1,265}, --The Slave Pens
	[261]={1,263}, --The Steamvault
	[262]={1,262}, --The Underbog
	-- BC // Raid
	[745]={1,350}, --Karazhan
	[746]={1,330}, --Gruul's Lair
	[747]={1,331}, --Magtheridon's Lair
	[748]={1,332}, --Serpentshrine Cavern
	[749]={1,334}, --The Eye
	[750]={1,329}, --The Battle for Mount Hyjal
	[751]={1,339}, --Black Temple
	[752]={1,335}, --Sunwell Plateau

	-- WotlK // Dungeon
	[271]={2,132}, --Ahn'kahet: The Old Kingdom
	[272]={2,157}, --Azjol-Nerub
	[273]={2,160}, --Drak'Tharon Keep
	[274]={2,154}, --Gundrak
	[275]={2,138}, --Halls of Lightning
	[276]={2,185}, --Halls of Reflection
	[277]={2,140}, --Halls of Stone
	[278]={2,184}, --Pit of Saron
	[279]={2,130}, --The Culling of Stratholme
	[280]={2,183}, --The Forge of Souls
	[281]={2,129}, --The Nexus
	[282]={2,143}, --The Oculus
	[283]={2,168}, --The Violet Hold
	[284]={2,171}, --Trial of the Champion
	[285]={2,133}, --Utgarde Keep
	[286]={2,136}, --Utgarde Pinnacle
	-- WotlK // Raid
	[753]={2,156}, --Vault of Archavon
	[754]={2,162}, --Naxxramas
	[755]={2,155}, --The Obsidian Sanctum
	[756]={2,141}, --The Eye of Eternity
	[757]={2,172}, --Trial of the Crusader
	[758]={2,186}, --Icecrown Citadel
	[759]={2,147}, --Ulduar
	[760]={2,248}, --Onyxia's Lair
	[761]={2,200}, --The Ruby Sanctum

	-- Cata // Dungeon
	["63:2"]={3,291}, --Deadmines(Heroic)
	["64:2"]={3,310}, --Shadowfang Keep(Heroic)
	[65]={3,322}, --Throne of the Tides
	[66]={3,283}, --Blackrock Caverns
	[67]={3,324}, --The Stonecore
	[68]={3,325}, --The Vortex Pinnacle
	[69]={3,277}, --Lost City of the Tol'vir
	[70]={3,297}, --Halls of Origination
	[76]={3,233}, --Zul'Gurub
	[77]={3,333}, --Zul'Aman
	[184]={3,401}, --End Time
	[185]={3,398}, --Well of Eternity
	[186]={3,294}, --Hour of Twilight
	-- Cata // Raid
	[75]={3,282}, --Baradin Hold
	[73]={3,285}, --Blackwing Descent
	[72]={3,399}, --The Bastion of Twilight
	[74]={3,328}, --Throne of the Four Winds
	[78]={3,367}, --Firelands
	[187]={3,409}, --Dragon Soul

	-- MoP // Dungeon
	["246:2"]={4,476}, --Scholomance
	[302]={4,439}, --Stormstout Brewery
	[303]={4,437}, --Gate of the Setting Sun
	["311:2"]={4,431}, --Scarlet Halls
	[312]={4,443}, --Shado-Pan Monastery
	[313]={4,429}, --Temple of the Jade Serpent
	["316:2"]={4,302}, --Scarlet Monastery
	[321]={4,453}, --Mogu'shan Palace
	[324]={4,457}, --Siege of Niuzao Temple
	-- MoP // Raid
	--[322]={4,0}, --Pandaria-WorldBosses
	[317]={4,471}, --Mogu'shan Vaults
	[330]={4,474}, --Heart of Fear
	[320]={4,456}, --Terrace of Endless Spring
	[362]={4,508}, --Throne of Thunder
	[369]={4,556}, --Siege of Orgrimmar

	-- WoD // Dungeon
	[385]={5,573}, --Bloodmaul Slag Mines
	[476]={5,601}, --Skyreach
	[536]={5,606}, --Grimrail Depot
	[537]={5,574}, --Shadowmoon Burial Grounds
	[547]={5,593}, --Auchindoun
	[556]={5,620}, --The Everbloom
	--[557]={5,0}, --Draenor-WorldBosses
	[558]={5,595}, --Iron Docks
	[559]={5,616}, --Upper Blackrock Spire
	-- WoD // Raid
	[477]={5,610}, --Highmaul
	[457]={5,596}, --Blackrock Foundry
	[669]={5,661}, --Hellfire Citadel

	-- Legion // Dungeon
	[707]={6,677}, --Vault of the Wardens
	[716]={6,713}, --Eye of Azshara
	[721]={6,703}, --Halls of Valor
	[726]={6,749}, --The Arcway
	[727]={6,706}, --Maw of Souls
	[740]={6,751}, --Black Rook Hold
	[762]={6,733}, --Darkheart Thicket
	[767]={6,731}, --Neltharion's Lair
	[777]={6,732}, --Assault on Violet Hold
	[786]={6,764}, --The Nighthold
	[800]={6,761}, --Court of Stars
	[860]={6,350}, --Return to Karazhan
	-- Legion // Raid
	--[822]={6,0}, --Broken Isles-WorldBosses
	[768]={6,777}, --The Emerald Nightmare
	[861]={6,806}, --Trial of Valor
	[875]={6,850}, --Tomb of Sargeras
	[900]={6,845}, --Cathedral of Eternal Night

	-- BfA // Dungeon
	[968]={7,934}, --Atal'Dazar
	[1001]={7,936}, --Freehold
	[1002]={7,975}, --Tol Dagor
	[1012]={7,1010}, --The MOTHERLODE!!
	[1021]={7,1015}, --Waycrest Manor
	[1022]={7,1042}, --The Underrot
	[1023]={7,1162}, --Siege of Boralus
	[1030]={7,1038}, --Temple of Sethraliss
	[1036]={7,1040}, --Shrine of the Storm
	[1178]={7,1490}, --Operation: Mechagon
	-- BfA // Raid
	--[1028]={7,0}, --Azeroth-WorldBosses
	[1031]={7,1148}, --Uldir
	[1176]={7,1352}, --Battle of Dazar'alor
	[1177]={7,1345}, --Crucible of Storms
	[1179]={7,1512}, --The Eternal Palace
	[1180]={7,1580}, --Ny'alotha, the Waking City

	-- The Shadowlands // Dungeon
	[1182]={8,1666}, --The Necrotic Wake
	[1183]={8,1674}, --Plaguefall
	[1184]={8,1669}, --Mists of Tirna Scithe
	[1185]={8,1663}, --Halls of Atonement
	[1186]={8,1662}, --Spires of Ascension
	[1187]={8,1683}, --Theater of Pain
	[1188]={8,1680}, --De Other Side
	[1189]={8,1675}, --Sanguine Depths
	-- The Shadowlands // Raid
	--[1192]={8,0}, --WorldBosses
	[1190]={8,1735}, --Castle Nathria
	[1195]={8,2061}, --Sepulcher of the First Ones

	-- Dragonflight // Dungeon
	[1196]={9,2106}, --Brackenhide Hollow
	[1197]={9,000}, --Uldaman: Legacy of Tyr
	[1198]={9,2093}, --The Nokhud Offensive
	[1199]={9,2080}, --Neltharus
	--[1200]={9,000}, --Vault of the Incarnates
	[1201]={9,2097}, --Algeth'ar Academy
	[1202]={9,2094}, --Ruby Life Pools
	[1203]={9,000}, --The Azure Vault
	[1204]={9,2082}, --Halls of Infusion

	-- Dragonflight // Raid
	--[1205]={9,0}, --WorldBosses
	[1206]={9,000}, --The Nokud Offensive
}

-- sometimes there are bugged ID's so we fix it here
GGL.FIX_MAPID_TO_INSTANCID = {
	[256] = 247, -- Auchenai Crypts
	[302] = 316, -- Scarlet Monastery / FH
	[845] = 900, -- Cathedral of Eternal Night
	[319] = 744, --Temple of Ahn'Qiraj
}

for iniID, iniData in pairs(GGL.INSTANCE_ID_DATA) do
	if type(iniID) == "number" and iniData[2] > 0 then
		GGL.FIX_MAPID_TO_INSTANCID[iniData[2]] = iniID
	end
end

GGL.MAP_BASE_ID = {
	[74]=71,[327]=81,[1163]=1165,[19]=18,[16]=15,[715]=619,[86]=85,[1579]=1530,[33]=36,[577]=534,
	[1504]=1355,[1528]=1355,

	[2]=1,[3]=1,[4]=1,[5]=1,[6]=1,[8]=7,[9]=7,[11]=10,[16]=15,[19]=18,
	[20]=18,[24]=23,[28]=27,[29]=27,[30]=27,[31]=27,[33]=36,[34]=36,[35]=32,[38]=37,
	[39]=37,[40]=37,[41]=42,[43]=42,[44]=42,[45]=42,[46]=42,[53]=52,[54]=52,[55]=52,
	[58]=57,[59]=57,[60]=57,[61]=57,[67]=66,[68]=66,[72]=71,[73]=71,[74]=71,[75]=71,
	[79]=78,[82]=81,[86]=85,[91]=25,[92]=10,[93]=14,[96]=95,[98]=97,[99]=97,[112]=109,
	[124]=23,[126]=125,[128]=115,[131]=130,[134]=133,[135]=133,[137]=136,[139]=138,[142]=114,[144]=143,
	[145]=143,[146]=143,[148]=147,[149]=147,[150]=147,[151]=147,[152]=147,[158]=157,[159]=157,[161]=160,
	[163]=162,[164]=162,[165]=162,[166]=162,[167]=162,[169]=118,[173]=172,[175]=174,[176]=174,[177]=174,
	[178]=174,[182]=181,[187]=186,[188]=186,[189]=186,[190]=186,[191]=186,[192]=186,[193]=186,[195]=194,
	[196]=194,[197]=194,[206]=241,[208]=207,[209]=207,[218]=217,[222]=221,[223]=221,[227]=226,[228]=226,
	[229]=226,[231]=230,[234]=69,[236]=235,[237]=235,[238]=235,[239]=235,[240]=235,[243]=242,[251]=250,
	[252]=250,[253]=250,[254]=250,[255]=250,[257]=256,[259]=258,[264]=263,[268]=267,[270]=269,[271]=269,
	[281]=280,[284]=283,[286]=285,[288]=287,[289]=287,[290]=287,[292]=291,[295]=294,[296]=294,[298]=297,
	[299]=297,[303]=302,[304]=302,[305]=302,[307]=306,[308]=306,[309]=306,[311]=310,[312]=310,[313]=310,
	[314]=310,[315]=310,[316]=310,[318]=317,[320]=319,[321]=319,[323]=322,[336]=335,[337]=50,[338]=198,
	[340]=339,[341]=339,[342]=339,[343]=339,[344]=339,[345]=339,[346]=339,[349]=348,[351]=350,[352]=350,
	[353]=350,[354]=350,[355]=350,[356]=350,[357]=350,[358]=350,[359]=350,[360]=350,[361]=350,[362]=350,
	[363]=350,[364]=350,[365]=350,[366]=350,[368]=367,[369]=367,[370]=129,[372]=371,[373]=371,[374]=371,
	[375]=371,[377]=376,[380]=379,[381]=379,[382]=379,[383]=379,[384]=379,[385]=379,[386]=379,[387]=379,
	[389]=388,[391]=390,[392]=390,[393]=390,[394]=390,[395]=390,[396]=390,[397]=109,[400]=399,[402]=401,
	[403]=401,[404]=401,[405]=401,[406]=401,[410]=409,[411]=409,[412]=409,[413]=409,[414]=409,[415]=409,
	[416]=70,[417]=390,[419]=418,[420]=418,[421]=418,[426]=37,[427]=27,[428]=27,[430]=429,[432]=431,
	[434]=379,[435]=302,[436]=302,[438]=437,[440]=439,[441]=439,[442]=439,[444]=443,[445]=443,[446]=443,
	[447]=371,[448]=371,[449]=390,[450]=418,[451]=422,[452]=379,[454]=453,[455]=453,[458]=457,[459]=457,
	[464]=1,[466]=18,[470]=27,[472]=471,[473]=471,[475]=474,[477]=476,[478]=476,[479]=476,[482]=481,
	[483]=70,[486]=418,[487]=418,[488]=433,[490]=339,[491]=339,[492]=339,[493]=339,[494]=339,[495]=339,
	[496]=339,[497]=339,[498]=418,[500]=499,[501]=125,[502]=125,[505]=504,[506]=504,[509]=508,[510]=508,
	[511]=508,[512]=508,[513]=508,[514]=508,[515]=508,[517]=505,[519]=376,[520]=390,[523]=27,[526]=525,
	[527]=525,[528]=525,[529]=525,[530]=525,[531]=525,[532]=525,[533]=525,[536]=535,[537]=535,[538]=535,
	[539]=104,[540]=539,[541]=539,[544]=543,[545]=543,[546]=543,[547]=543,[548]=543,[549]=543,[550]=107,
	[551]=550,[552]=550,[553]=550,[555]=554,[557]=556,[558]=556,[559]=556,[560]=556,[561]=556,[562]=556,
	[563]=556,[564]=556,[565]=556,[566]=556,[567]=556,[568]=556,[569]=556,[570]=556,[571]=554,[575]=574,
	[576]=574,[579]=539,[580]=539,[581]=539,[582]=539,[585]=525,[586]=525,[587]=525,[589]=588,[590]=525,
	[594]=535,[597]=596,[598]=596,[599]=596,[600]=596,[602]=601,[607]=606,[608]=606,[609]=606,[611]=610,
	[612]=610,[613]=610,[614]=610,[615]=610,[617]=616,[618]=616,[621]=620,[622]=588,[623]=25,[624]=588,
	[626]=125,[627]=125,[628]=125,[629]=125,[631]=630,[632]=630,[633]=630,[635]=634,[636]=634,[637]=634,
	[638]=634,[639]=634,[640]=634,[642]=641,[643]=641,[644]=641,[648]=647,[649]=634,[651]=650,[652]=650,
	[653]=650,[654]=650,[655]=650,[656]=650,[657]=650,[658]=650,[659]=650,[660]=650,[662]=661,[663]=661,
	[664]=661,[665]=661,[666]=661,[667]=661,[668]=661,[669]=661,[670]=661,[675]=674,[678]=677,[679]=677,
	[681]=680,[682]=680,[683]=680,[684]=680,[685]=680,[686]=680,[687]=680,[688]=680,[689]=680,[690]=680,
	[691]=680,[692]=680,[693]=680,[697]=76,[698]=186,[699]=186,[700]=186,[701]=186,[704]=703,[705]=703,
	[707]=706,[708]=706,[709]=378,[710]=677,[711]=677,[712]=677,[715]=641,[718]=717,[720]=719,[721]=719,
	[723]=168,[726]=725,[728]=433,[729]=207,[735]=734,[737]=325,[738]=198,[741]=740,[743]=742,[744]=147,
	[745]=147,[746]=147,[747]=641,[748]=714,[750]=650,[752]=751,[753]=751,[754]=751,[755]=751,[756]=751,
	[757]=116,[758]=641,[759]=339,[760]=198,[762]=761,[763]=761,[765]=764,[766]=764,[767]=764,[768]=764,
	[769]=764,[770]=764,[771]=764,[772]=764,[778]=777,[779]=777,[780]=777,[781]=777,[782]=777,[783]=777,
	[784]=777,[785]=777,[786]=777,[787]=777,[788]=777,[789]=777,[791]=429,[792]=429,[793]=641,[794]=350,
	[795]=350,[796]=350,[797]=350,[798]=749,[799]=114,[800]=143,[801]=143,[802]=143,[803]=143,[804]=302,
	[805]=302,[807]=806,[808]=806,[809]=350,[810]=350,[811]=350,[812]=350,[813]=350,[814]=350,[815]=350,
	[816]=350,[817]=350,[818]=350,[819]=350,[820]=350,[821]=350,[822]=350,[823]=118,[825]=279,[827]=317,
	[828]=141,[829]=703,[831]=830,[832]=830,[833]=830,[834]=27,[835]=291,[836]=291,[837]=14,[838]=36,
	[840]=226,[841]=226,[842]=226,[843]=379,[844]=14,[846]=845,[847]=845,[848]=845,[849]=845,[851]=850,
	[852]=850,[853]=850,[854]=850,[855]=850,[856]=850,[857]=328,[859]=10,[860]=115,[861]=672,[866]=865,
	[870]=869,[872]=439,[873]=439,[874]=439,[877]=634,[879]=719,[880]=719,[881]=141,[883]=882,[884]=882,
	[886]=885,[887]=885,[890]=889,[892]=891,[893]=891,[894]=891,[898]=897,[899]=897,[900]=897,[901]=897,
	[902]=897,[903]=882,[904]=81,[910]=909,[911]=909,[912]=909,[913]=909,[914]=909,[915]=909,[916]=909,
	[917]=909,[918]=909,[919]=909,[920]=909,[935]=934,[941]=940,[943]=906,[972]=971,[974]=895,[976]=975,
	[977]=975,[978]=975,[979]=975,[980]=975,[997]=18,[998]=90,[1009]=864,[1016]=1015,[1017]=1015,[1018]=1015,
	[1029]=1015,[1030]=217,[1031]=181,[1043]=1038,[1044]=906,[1149]=1148,[1150]=1148,[1151]=1148,[1152]=1148,[1153]=1148,
	[1154]=1148,[1155]=1148,[1157]=1156,[1158]=14,[1159]=242,[1160]=242,[1163]=1165,[1164]=1165,[1167]=1166,[1171]=895,
	[1172]=895,[1173]=862,[1174]=862,[1176]=862,[1179]=942,[1180]=942,[1184]=895,[1185]=895,[1186]=242,[1332]=1203,
	[1333]=1203,[1334]=123,[1338]=1203,[1346]=1345,[1349]=895,[1350]=975,[1351]=975,[1353]=1352,[1354]=1352,[1356]=1352,
	[1357]=1352,[1358]=1352,[1359]=186,[1360]=186,[1362]=1039,[1363]=1345,[1364]=1352,[1366]=93,[1367]=1352,[1375]=140,
	[1381]=1148,[1382]=1148,[1383]=14,[1408]=588,[1465]=431,[1468]=641,[1471]=641,[1472]=650,[1473]=1021,[1475]=641,
	[1479]=390,[1491]=1490,[1493]=1490,[1494]=1490,[1497]=1490,[1499]=1355,[1505]=317,[1513]=1512,[1514]=1512,[1515]=1512,
	[1516]=1512,[1517]=1512,[1518]=1512,[1519]=1512,[1520]=1512,[1522]=1462,[1527]=249,[1530]=390,[1531]=194,[1532]=194,
	[1534]=86,[1537]=25,[1538]=285,[1539]=285,[1540]=297,[1541]=297,[1542]=297,[1544]=453,[1545]=453,[1546]=453,
	[1547]=471,[1548]=471,[1549]=471,[1553]=1552,[1554]=332,[1555]=334,[1556]=198,[1557]=162,[1558]=118,[1559]=241,
	[1560]=36,[1561]=198,[1563]=118,[1570]=520,[1574]=1573,[1576]=376,[1577]=202,[1578]=242,[1579]=1530,[1581]=1580,
	[1582]=1580,[1590]=1580,[1591]=1580,[1592]=1580,[1593]=1580,[1594]=1580,[1595]=1580,[1596]=1580,[1597]=1580,[1600]=1530,
	[1602]=186,[1610]=1609,[1615]=1543,[1616]=1543,[1617]=1543,[1618]=1543,[1619]=1543,[1620]=1543,[1621]=1543,[1623]=1543,
	[1624]=1543,[1627]=1543,[1628]=1543,[1629]=1543,[1630]=1543,[1631]=1543,[1632]=1543,[1635]=1543,[1636]=1543,[1641]=1543,
	[1644]=1525,[1649]=1536,[1650]=1536,[1651]=1536,[1652]=1536,[1664]=1663,[1665]=1663,[1667]=1666,[1668]=1666,[1671]=1670,
	[1672]=1670,[1673]=1670,[1676]=1675,[1678]=1677,[1679]=1677,[1680]=1677,[1681]=186,[1682]=186,[1684]=1683,[1685]=1683,
	[1686]=1683,[1687]=1683,[1690]=1533,[1693]=1692,[1694]=1692,[1695]=1692,[1697]=1674,[1700]=1699,[1702]=1701,[1703]=1701,
	[1708]=1707,[1709]=1565,[1711]=1533,[1712]=1543,[1713]=1533,[1714]=1533,[1721]=1543,[1727]=1726,[1736]=1543,[1744]=1735,
	[1745]=1735,[1746]=1735,[1747]=1735,[1748]=1735,[1749]=1543,[1750]=1735,[1751]=1543,[1752]=1543,[1753]=1543,[1754]=1543,
	[1755]=1525,[1756]=1543,[1757]=1543,[1758]=1543,[1759]=1543,[1760]=1543,[1761]=1543,[1763]=1543,[1764]=1543,[1765]=1543,
	[1766]=1543,[1767]=1543,[1768]=1543,[1769]=1543,[1770]=1543,[1771]=1543,[1772]=1543,[1773]=1543,[1774]=1543,[1776]=1543,
	[1777]=1543,[1778]=1543,[1779]=1543,[1780]=1543,[1781]=1543,[1782]=1543,[1783]=1543,[1784]=1543,[1785]=1543,[1786]=1543,
	[1787]=1543,[1788]=1543,[1789]=1543,[1791]=1543,[1792]=1543,[1793]=1543,[1794]=1543,[1795]=1543,[1796]=1543,[1797]=1543,
	[1798]=1543,[1799]=1543,[1800]=1543,[1801]=1543,[1802]=1543,[1803]=1543,[1804]=1543,[1805]=1543,[1806]=1543,[1807]=1543,
	[1808]=1543,[1809]=1543,[1810]=1543,[1811]=1543,[1812]=1543,[1816]=1565,[1818]=1565,[1819]=1565,[1820]=1543,[1821]=1543,
	[1822]=1543,[1823]=1543,[1824]=1565,[1825]=1565,[1826]=1565,[1827]=1565,[1911]=1543,[1912]=1543,[1913]=1543,[1917]=1565,
	[1920]=1543,[1958]=367,[1959]=367,[1960]=1543,[1962]=1543,[1963]=1543,[1964]=1543,[1965]=1543,[1966]=1543,[1967]=1543,
	[1968]=1543,[1969]=1543,[1974]=1543,[1975]=1543,[1976]=1543,[1977]=1543,[1979]=1543,[1980]=1543,[1981]=1543,[1982]=1543,
	[1983]=1543,[1984]=1543,[1985]=1543,[1986]=1543,[1987]=1543,[1988]=1543,[1990]=1989,[1991]=1989,[1992]=1989,[1993]=1989,
	[1995]=1989,[1996]=1989,[1997]=1989,[1999]=1998,[2000]=1998,[2001]=1998,[2002]=1998,[2003]=1998,[2004]=1998,[2005]=1603,
	[2006]=1961,[2007]=1961,[2008]=1961,[2017]=1533,[2018]=1533,[2027]=1970,[2028]=1970,[2029]=1970,[2030]=1970,[2031]=1970,
	[2048]=2047,[2049]=2047,[2050]=2047,[2051]=2047,[2052]=2047,[2061]=2047,[2066]=1970,[2070]=18,[2085]=2025,[2095]=2094,
	[2098]=2097,[2099]=2097,[2106]=2096,[2109]=2107,[2110]=2107,[2111]=2107,[2118]=2107,[2132]=2024,

	-- override
	[539]=539,

	-- Shadowlands Covenant Sanctum
	[1707]=1533,[1708]=1533,[1701]=1565,[1702]=1565,[1703]=1565,[1699]=1525,[1707]=1525,[1698]=1536,

	-- Dalaran fixes
	[501]=25,[502]=25,[626]=626,[627]=626,[628]=626,[629]=626,
}
