local AddonName, GGL = ...

local Export = {}
GGL.Export = Export

-- lua
local type = type
local pairs = pairs
local concate = table.concat
local gsub = string.gsub

-- wow

-- other
local AceGUI = LibStub("AceGUI-3.0")
local L = GGL.Locale

-- ########################################
-- Base
-- ########################################
local ExportFrame
local function GetExportFrame()
    if not ExportFrame then
        local frame = AceGUI:Create("Frame")
        frame:SetTitle(L["Export"])
        --frame:SetStatusText(listName)
        frame:SetLayout("Fill")

        local multiEditbox = AceGUI:Create("MultiLineEditBox")
        multiEditbox.frame:SetClipsChildren(true)
        --multiEditbox.editBox:SetMaxBytes(nil)
        multiEditbox.editBox:SetScript("OnEscapePressed", function()
            frame:Hide()
        end)
        multiEditbox.editBox:SetScript("OnMouseUp", function()
            multiEditbox.editBox:HighlightText()
        end)
        multiEditbox.editBox:SetScript("OnChar", function(self)
            multiEditbox:SetText(self.exportText)
            multiEditbox.editBox:HighlightText()
        end)
        frame.multiEditbox = multiEditbox
        frame:AddChild(multiEditbox)

        ExportFrame = frame
    end
    ExportFrame:Show()
    return ExportFrame
end

function Export:FillExportWindow(data, exportFormat, ...)
    exportFormat = exportFormat or "csv"

    local multiEditbox = GetExportFrame().multiEditbox

    local exportText
    if exportFormat == "csv" then
        local delimiter, noHeader = ...
        exportText = GGL.Export:ExportAsCSV(data, delimiter or ",", noHeader == nil and true or noHeader)
    elseif exportFormat == "text" then
        exportText = data
    end

    multiEditbox.editBox.exportText = exportText or ""
    multiEditbox:SetText(exportText or "")

    multiEditbox.editBox:HighlightText()
    multiEditbox.editBox:SetFocus()
end

-- ########################################
-- csv
-- ########################################

local function CSV_FormatStringWithDelimiter(str, delimiter)
    if not delimiter or type(str) ~= "string" then
        return str
    end

    return gsub(str, delimiter, "")
end

local function CSV_FormatTableStrings(tab, delimiter)
    if not tab then return end
    local newTab = {}
    for i = 1, #tab do
        newTab[i] = CSV_FormatStringWithDelimiter(tab[i], delimiter)
    end
    return concate(newTab, delimiter)
end

--- Get a csv string
-- tab = {
--      header = {"pos1", "pos2"},  -- header
--      {1,2} -- index rows
-- }
-- @param       delimiter       <string>
function Export:ExportAsCSV(tab, delimiter, noHeader)
    if not tab then return end

    delimiter = delimiter or ","

    local csv = {}

    -- check if we have header
    if not noHeader and tab.header then
        csv[1] = CSV_FormatTableStrings(tab.header, delimiter)
    end

    -- get entrys
    if #tab and #tab > 0 then
        for i = 1, #tab do
            csv[#csv+1] = CSV_FormatTableStrings(tab[i], delimiter)
        end
    else
        for k,v in pairs(tab) do
            csv[#csv+1] = CSV_FormatTableStrings({k, v}, delimiter)
        end
    end

    return concate(csv, "\n")
end