-- ###########################
-- EVENTS:
--	"Loot_OnItemLoot" - Loot, itemID, ammount, source, sourceID, mainSourceFlag, subSourceFlag, categoryFlag
-- 	"Loot_OnMoneyLoot" - Loot, ammount, source, sourceID, mainSourceFlag, subSourceFlag, categoryFlag
-- ###########################

local AddonName, GGL = ...

--local print = function() end

local Loot = {}
GGL.Loot = Loot

local GGL_GetTime = GGL.GetTime

local db, SessionDb

-- ###########################
-- WoW
-- ###########################
local LOOT_SLOT_NONE, LOOT_SLOT_ITEM, LOOT_SLOT_MONEY, LOOT_SLOT_CURRENCY
if _G.Enum and _G.Enum.LootSlotType then
	LOOT_SLOT_NONE = _G.Enum.LootSlotType.None or 0
	LOOT_SLOT_ITEM = _G.Enum.LootSlotType.Item or 1
	LOOT_SLOT_MONEY = _G.Enum.LootSlotType.Money or 2
	LOOT_SLOT_CURRENCY = _G.Enum.LootSlotType.Currency or 3
else
	LOOT_SLOT_NONE, LOOT_SLOT_ITEM, LOOT_SLOT_MONEY, LOOT_SLOT_CURRENCY= _G.LOOT_SLOT_NONE, _G.LOOT_SLOT_ITEM, _G.LOOT_SLOT_MONEY, _G.LOOT_SLOT_CURRENCY
end

local GetContainerNumSlots = (C_Container and C_Container.GetContainerNumSlots) and C_Container.GetContainerNumSlots or GetContainerNumSlots
local GetContainerItemID = (C_Container and C_Container.GetContainerItemID) and C_Container.GetContainerItemID or GetContainerItemID
local GetContainerItemInfo = (C_Container and C_Container.GetContainerItemInfo) and C_Container.GetContainerItemInfo or GetContainerItemInfo
local GetContainerItemLink = (C_Container and C_Container.GetContainerItemLink) and C_Container.GetContainerItemLink or GetContainerItemLink

-- ###########################
-- Lua
-- ###########################
local assert, type, select, wipe =
	  assert, type, select, wipe
local pairs, ipairs =
	  pairs, ipairs
local str_find, str_match, str_format, str_split, strsplit =
	  string.find, string.match, string.format, string.split, strsplit

local bit_band =
	  bit.band


local GetItemInfoInstant, GetLootSlotInfo, GetLootSlotLink, GetLootSlotType, GetLFGCompletionReward, GetLFGCompletionRewardItem =
	  GetItemInfoInstant, GetLootSlotInfo, GetLootSlotLink, GetLootSlotType, GetLFGCompletionReward, GetLFGCompletionRewardItem
local IsWorldQuest = GGL.IS_RETAIL and C_QuestLog.IsWorldQuest or function() return false end

-- ###########################
-- FLAGS
-- ###########################
local MAIN_SOURCE_FLAG = GGL.MAIN_SOURCE_FLAG

local SUB_SOURCE_FLAG = GGL.SUB_SOURCE_FLAG

local CATEGORY_FLAG = GGL.CATEGORY_FLAG

-- ###########################
-- GGL
-- ###########################
local LOOT_SPECIAL_CAST_ITEMS = GGL.LOOT_SPECIAL_CAST_ITEMS

-- ###########################
-- Local
-- ###########################
local EventFrame_OnUpdateSender
local itemIconCache = {[0] = 0}
local BagLootIsOpen = false
local PlayerGUID, PlayerName
local LastUnitTarget, LastLootTarget, LastTargetSkip

local LOOT_TIMEOUT = 600
local LOOT_SOURCE_DATA = {
	itemList = {},
	slotItemList = {},
	slotItemListSave = {},
	lootedList = {},
}

GGL:AddCallbackHandler(Loot)

-- ########################################
-- Parser
-- ########################################
-- Parse the GUID type
local GUID_TYPE_LIST = {
	["Creature"] 	=	{ idPos = 6,	mainSourceType = GGL.MAIN_SOURCE_FLAG.NPC },		-- [unitType]-0-[serverID]-[instanceID]-[zoneUID]-[ID]-[spawnUID]
	["GameObject"] 	=	{ idPos = 6,	mainSourceType = GGL.MAIN_SOURCE_FLAG.OBJECT },		-- [unitType]-0-[serverID]-[instanceID]-[zoneUID]-[ID]-[spawnUID]
	["Vehicle"] 	=	{ idPos = 6,	mainSourceType = GGL.MAIN_SOURCE_FLAG.NPC },		-- [unitType]-0-[serverID]-[instanceID]-[zoneUID]-[ID]-[spawnUID]
	["Pet"] 		=	{ idPos = 6,	mainSourceType = GGL.MAIN_SOURCE_FLAG.NPC },		-- [unitType]-0-[serverID]-[instanceID]-[zoneUID]-[ID]-[spawnUID]
	["Item"] 		=	{ idPos = 0,	mainSourceType = GGL.MAIN_SOURCE_FLAG.ITEM },		-- Item-[serverID]-0-[spawnUID]
}

--Creature-0-1463-2241-22016-162374-00002DA785
--Creature-0-1463-2241-22016-160464-00002EF874
-- @returns		mainSourceFlag, sourceID
local function ParseGUID(guid)
	if not guid then return end

	local guidData = {strsplit("-", guid)}
	if guidData[1] and GUID_TYPE_LIST[guidData[1]] then
		return GUID_TYPE_LIST[guidData[1]].mainSourceType, guidData[GUID_TYPE_LIST[guidData[1]].idPos]
	end
end

-- Parse the ammount of Copper from the money loot
local PARSE_MONEY_AMOUNT = {
	{COPPER_AMOUNT:gsub("%%d", "(%%d+)"), 1},
	{SILVER_AMOUNT:gsub("%%d", "(%%d+)"), 100},
	{GOLD_AMOUNT:gsub("%%d", "(%%d+)"), 10000},
}
local function ParseMoneyCopper(moneyString)
	if not moneyString then return end
	local coins = 0
	local money,_
	for i = 1, #PARSE_MONEY_AMOUNT do
		_,_,money = str_find(moneyString, PARSE_MONEY_AMOUNT[i][1])
		if tonumber(money or "") then
			coins = coins + (tonumber(money) * PARSE_MONEY_AMOUNT[i][2])
		end
	end
	return coins
end

-- Parse a Tooltip for a specific text
local function ParseToolTipForText(tt, text)
	if not tt or not text then return end
	local tt_name = tt:GetName().."TextLeft"
	local ttText
	for i=2, tt:NumLines() do
		ttText = _G[tt_name..i]:GetText()
		if ttText == text then
			return true
		end
	end
end

local ItemLinkCache = GGL:GetCacheTable()
local ITEM_BONUS_PARSE_STRING = "|Hitem:(%d+):%d*:%d*:%d*:%d*:%d*:%d*:%d*:%d*:%d*:%d*:%d*:(.*)|h%["
local function GetItemIDData(itemLink)
	if not itemLink then return end
	if ItemLinkCache[itemLink] then return ItemLinkCache[itemLink] end
	local itemID, itemBonus = str_match(itemLink, ITEM_BONUS_PARSE_STRING)
	itemID = itemID or GetItemInfoInstant(itemLink)
	itemID = tonumber(itemID)
	if itemBonus then
		local data = { str_split(":", itemBonus) }
		if data then
			-- check if we have any bonus things
			local pass = false
			for i = 1, #data do
				if data[i] and data[i] ~= "" and data[i] ~= "0" then
					pass = true
					break
				end
			end
			-- add bonus
			if pass then
				itemID = itemID..","..itemBonus
			end
		end
	end
	ItemLinkCache[itemLink] = itemID
	return itemID
end

-- ###########################
-- Base
-- ###########################
local FORMAT_PUSH_STRING = {
	item = "%s x %d",
}
local FORMAT_PUSH_FUNCTION = {
	item = function(itemID, qty)
		local _, itemLink = GetItemInfo(itemID)
		return str_format(FORMAT_PUSH_STRING.item, itemLink, qty)
	end,
	money = function(copper)
		return copper
	end
}

local timeOffset = nil;
-- returns a unix timestamp in milliseconds
local function GetTimeNow()
    -- Init
    if not timeOffset then
        timeOffset = floor((GetServerTime() - GetTime()) * 1000);
    end

    return timeOffset + floor(GetTime() * 1000);
end

local function IsValidInterval(value1, value2, delta)
	local bandwidthIn, bandwidthOut, latencyHome = GetNetStats()
    return abs(value1 - value2) <= (latencyHome + delta)
end

-- ###########################
-- Loot Push
-- ###########################
local function GetSourcesFromLoot(data, itemID, ammount)
	if not data[itemID] then return end
	--local print = itemID==0 and function() end or _G.print
	local source
	if data[itemID].fullLootQuant <= ammount then
		--print("Loot all", itemID, ammount, data[itemID].fullLootQuant)
		source = data[itemID]
		data[itemID] = nil
	else
		source = {fullLootQuant = 0}
		for i, lootData in ipairs(data[itemID]) do
			if lootData ~= true then
				ammount = ammount - lootData[5]

				if ammount >= 0 then
					--print("Full npc loot", itemID, lootData[5], ammount)
					source[#source+1] = lootData
					data[itemID].fullLootQuant = data[itemID].fullLootQuant - lootData[5]
					source.fullLootQuant = source.fullLootQuant + lootData[5]
					data[itemID][i] = true
				else
					--print("Part npc loot", itemID, lootData[5], ammount)
					local fixAmmount = ammount + lootData[5]
					source[#source+1] = {unpack(lootData)}
					source[#source][5] = source[#source][5] - fixAmmount
					source.fullLootQuant = source.fullLootQuant + fixAmmount
					data[itemID].fullLootQuant = data[itemID].fullLootQuant - fixAmmount
					lootData[5] = lootData[5] - fixAmmount
				end
				if ammount <= 0 then
					--print("End of Loot", itemID, ammount)
					break
				end
			end
		end
		if data[itemID].fullLootQuant <= 0 then
			data[itemID] = nil
		end
	end
	return source
end

local function PushItemLootFromLootedList(itemID, ammount, itemLog)
	local itemListData = LOOT_SOURCE_DATA.lootedList[itemID]

	if itemListData then
		local sources = GetSourcesFromLoot(LOOT_SOURCE_DATA.lootedList, itemID, ammount)
		if sources then
			Loot:FireCallbackEvent("Loot_OnItemLoot", Loot, itemID, itemLog, ammount, sources)
		end
	end
end

local function PushMoneyLootFromLootedList(ammount)
	local moneyLootData = LOOT_SOURCE_DATA.lootedList[0]

	if moneyLootData and ammount and ammount > 0 then
		local sources = GetSourcesFromLoot(LOOT_SOURCE_DATA.lootedList, 0, ammount)
		if sources then
			Loot:FireCallbackEvent("Loot_OnMoneyLoot", Loot, ammount, sources)
		end
	end
end

local function PushMoneyLootDirect(ammount, sourceID, mainSourceFlag, subSourceFlag, categoryFlag)
	if not ammount or ammount <= 0 then return end
	local sources = {
		{
			mainSourceFlag or MAIN_SOURCE_FLAG.NONE,
			subSourceFlag or LOOT_SOURCE_DATA.NONE,
			categoryFlag or CATEGORY_FLAG.LOOT,
			sourceID or 0,
			ammount or 1
		}
	}

	Loot:FireCallbackEvent("Loot_OnMoneyLoot", Loot, ammount, sources)
end

-- ###########################
-- LOOT_SOURCE_DATA
-- Handles the clear of the source flags
-- ###########################
local UpdateFrame = CreateFrame("Frame")
local LootSaverOnUpdate

-- Clear all source flags
local function ClearLootSourceSaver(delay)
	if delay then
		UpdateFrame.delay = delay
		UpdateFrame:SetScript("OnUpdate", LootSaverOnUpdate)
	else
		if UpdateFrame.delay then
			UpdateFrame.delay = nil
			UpdateFrame:SetScript("OnUpdate", nil)
		end

		LOOT_SOURCE_DATA.mainSource = nil
		LOOT_SOURCE_DATA.subSourceFlag = nil
		LOOT_SOURCE_DATA.categoryFlag = nil
        LOOT_SOURCE_DATA.itemID = nil
        LOOT_SOURCE_DATA.time = nil

		LastTargetSkip = nil
	end
end

function LootSaverOnUpdate(self, elapsed)
    if self.delay then
        self.delay = self.delay - elapsed
        if self.delay <= 0 then
            ClearLootSourceSaver()
        end
	end
end

-- ###########################
-- LootDataClear
-- Handles the clear of the source flags
-- ###########################
local LootDataClearFrame = CreateFrame("Frame")
local LootDataClear_OnUpdate
local LootDataClear_Delay

local function LootDataClear_NewLoot()
	-- hold data for 15 seconds
	-- as there is a unit guide saver in the background no double entrys should happen
	LootDataClear_Delay = 15
end

function LootDataClear_OnUpdate(self, elapsed)
	if LootDataClear_Delay then
		LootDataClear_Delay = LootDataClear_Delay - elapsed
		if LootDataClear_Delay <= 0 then
            wipe(LOOT_SOURCE_DATA.slotItemListSave)
			LootDataClear_Delay = nil
        end
	end
end
LootDataClearFrame:SetScript("OnUpdate", LootDataClear_OnUpdate)

-- ###########################
-- Events
-- ###########################
local EventFunctions = {}
local EventFrame = CreateFrame("Frame")
local function OnEvent(self, event, ...)
	EventFunctions[event](...)
end
EventFrame:SetScript("OnEvent", OnEvent)

-- list of used Events
local EVENT_LIST = {
	--"UNIT_SPELLCAST_START",
	"UNIT_SPELLCAST_SUCCEEDED",
	"LOOT_READY",
	"LOOT_CLOSED",
	"CHAT_MSG_LOOT",
	"CHAT_MSG_MONEY",
	"ISLAND_COMPLETED",
	"GROUP_LEFT",
	"LFG_COMPLETION_REWARD",
	"QUEST_TURNED_IN",
	"LOOT_SLOT_CLEARED",
	"ITEM_PUSH",
	"PLAYER_TARGET_CHANGED"
}

if GGL.IS_RETAIL then
	EVENT_LIST[#EVENT_LIST+1] = "SHOW_LOOT_TOAST"
	EVENT_LIST[#EVENT_LIST+1] = "QUEST_LOOT_RECEIVED"
	-- Mission table events
	EVENT_LIST[#EVENT_LIST+1] = "GARRISON_MISSION_BONUS_ROLL_COMPLETE"
	EVENT_LIST[#EVENT_LIST+1] = "GARRISON_MISSION_BONUS_ROLL_LOOT"
	EVENT_LIST[#EVENT_LIST+1] = "GARRISON_MISSION_NPC_OPENED"
	EVENT_LIST[#EVENT_LIST+1] = "GARRISON_MISSION_COMPLETE_RESPONSE"
end

for i = 1, #EVENT_LIST do
	EventFrame:RegisterEvent(EVENT_LIST[i])
end

local function MoveLootSlotToLootedList(slot, itemList)
	itemList = itemList or LOOT_SOURCE_DATA.slotItemList
	if itemList[slot] then
		local itemID = itemList[slot].itemID
		if not LOOT_SOURCE_DATA.lootedList[itemID] then
			LOOT_SOURCE_DATA.lootedList[itemID] = {
				fullLootQuant = 0
			}
		end
		local itemIDData = LOOT_SOURCE_DATA.lootedList[itemID]
		itemIDData.fullLootQuant = itemIDData.fullLootQuant + itemList[slot].fullLootQuant

		for i, lootData in ipairs(itemList[slot]) do
			itemIDData[#itemIDData+1] = lootData
		end

		itemList[slot] = false
	end
end

local function MoveSpecialLootSlotToLootedList(sourceType, itemID, ammount, sourceID, mainSourceFlag, subSourceFlag, categoryFlag)
	if not itemID then return end

	if not LOOT_SOURCE_DATA.lootedList[itemID] then
		LOOT_SOURCE_DATA.lootedList[itemID] = {
			fullLootQuant = 0
		}
	end

	LOOT_SOURCE_DATA.lootedList[itemID][#LOOT_SOURCE_DATA.lootedList[itemID]+1] = {
		mainSourceFlag or MAIN_SOURCE_FLAG.NONE,
		subSourceFlag or LOOT_SOURCE_DATA.NONE,
		categoryFlag or CATEGORY_FLAG.LOOT,
		sourceID or 0,
		ammount or 1
	}
end

local function ProcessLootSourcesBySlot(slot)
	-- { sourceGUID1, quant1, sourceGUID2, quant2 }
	local lootSourceInfo = { GetLootSourceInfo(slot) }
	local slotType = GetLootSlotType(slot)

	-- get itemID for money its 0
	local itemID
	if slotType == LOOT_SLOT_MONEY then
		itemID = 0
	else
		local _itemID, _, _, _, itemIcon = GetItemInfoInstant(GetLootSlotLink(slot))
		itemID = _itemID
		itemIconCache[itemIcon] = itemID
	end
	if not itemID then return end

	-- add guide saver
	LOOT_SOURCE_DATA.slotItemList.unitGUIDs = LOOT_SOURCE_DATA.slotItemList.unitGUIDs or {}
	-- check full ammount for loot slot
	local _, _, fullLootQuant = GetLootSlotInfo(slot)
	-- add slot into list
	LOOT_SOURCE_DATA.slotItemList[slot] = {
		itemID = itemID,
		fullLootQuant = 0,
	}
	local slotData = LOOT_SOURCE_DATA.slotItemList[slot]
	-- sourceGUID, quant
	for i = 1, #lootSourceInfo, 2 do
		local sourceGUID, quant = lootSourceInfo[i], lootSourceInfo[i+1]
		if LastTargetSkip or not LOOT_SOURCE_DATA.slotItemListSave.guideList or not LOOT_SOURCE_DATA.slotItemListSave.guideList[sourceGUID] then
			-- get mainSourceFlag and sourceID from GUID
			LOOT_SOURCE_DATA.slotItemList.unitGUIDs[sourceGUID] = true
			local mainSourceFlag, sourceID = ParseGUID(sourceGUID)
			local subSourceFlag

			-- check if the source is an item / most of the time this means opening
			if mainSourceFlag == MAIN_SOURCE_FLAG.ITEM and (not LOOT_SOURCE_DATA.subSourceFlag or LOOT_SOURCE_DATA.subSourceFlag == MAIN_SOURCE_FLAG.NONE) then
				subSourceFlag = SUB_SOURCE_FLAG.OPENING
				mainSourceFlag = MAIN_SOURCE_FLAG.ITEM
			end

			-- add data into slotData
			-- mainSourceFlag, subSourceFlag, categoryFlag, sourceID, quant
			slotData[#slotData+1] = {
				mainSourceFlag or LOOT_SOURCE_DATA.mainSource or MAIN_SOURCE_FLAG.NONE,
				subSourceFlag or LOOT_SOURCE_DATA.subSourceFlag or LOOT_SOURCE_DATA.NONE,
				LOOT_SOURCE_DATA.categoryFlag or CATEGORY_FLAG.LOOT,
				LOOT_SOURCE_DATA.itemID or sourceID,
				quant or 1
			}
			slotData.fullLootQuant = slotData.fullLootQuant + (quant or 1)
		else
			fullLootQuant = fullLootQuant - quant
		end
	end
	-- sometimes the loot count peer mob is wrong
	-- just add the missing quant to the last mob or we cant work with it
	if fullLootQuant > slotData.fullLootQuant and #slotData > 0 then
		slotData[#slotData][5] = slotData[#slotData][5] + (fullLootQuant - slotData.fullLootQuant)
		slotData.fullLootQuant = fullLootQuant
		-- print("fix quant", slot, itemID, fullLootQuant, slotData.fullLootQuant, slotData[#slotData][5])
	end
end

-- Loot Ready
local lootRedayDelay
function EventFunctions.LOOT_READY()
	LastLootTarget = LastUnitTarget
	-- set sourceInfo
	if IsFishingLoot() then
		-- check if its not fishing so we not override something here
		if LOOT_SOURCE_DATA.mainSource ~= SUB_SOURCE_FLAG.FISHING then
			LOOT_SOURCE_DATA.mainSource 	= MAIN_SOURCE_FLAG.ZONE
			LOOT_SOURCE_DATA.subSourceFlag 	= SUB_SOURCE_FLAG.FISHING
			LOOT_SOURCE_DATA.categoryFlag 	= CATEGORY_FLAG.PROFESSION + CATEGORY_FLAG.FARM_PROFESSION
			LOOT_SOURCE_DATA.itemID 		= nil
			LastTargetSkip 					= true
		end
	elseif LastTargetSkip then
		-- do nothing, gets cleared on end
	elseif LOOT_SOURCE_DATA.time and IsValidInterval(LOOT_SOURCE_DATA.time or 0, GetTimeNow(), 2000) then
		-- still valid source
	elseif BagLootIsOpen then
		-- no reset if BagLoot
	else
		-- clear as source from spell is no longer valid
		ClearLootSourceSaver()
	end

	-- check itemList
	for slot=1, GetNumLootItems() do
		local slotType = GetLootSlotType(slot)

		if slotType == LOOT_SLOT_NONE then
			-- nothing, skip
		elseif slotType == LOOT_SLOT_ITEM then
			-- item loot
			ProcessLootSourcesBySlot(slot)
		elseif slotType == LOOT_SLOT_MONEY then
			-- money loot
			ProcessLootSourcesBySlot(slot)
		elseif slotType == LOOT_SLOT_CURRENCY  then
			-- ignore currency, not needed
		end
	end

	-- perform mob loot
	GGL:LootMob()
end

-- Loot Window Closed
function EventFunctions.LOOT_CLOSED()
	----print("LOOT_CLOSED", GetTimeNow(), LOOT_SOURCE_DATA.time)
	LOOT_SOURCE_DATA.time = nil
    -- clear after a minute
	ClearLootSourceSaver(5)

	BagLootIsOpen = false

	if LastTargetSkip or (not LastUnitTarget or LastLootTarget ~= LastUnitTarget) and next(LOOT_SOURCE_DATA.slotItemList) then
		LOOT_SOURCE_DATA.slotItemListSave.guideList = LOOT_SOURCE_DATA.slotItemListSave.guideList or {}
		if LOOT_SOURCE_DATA.slotItemList and LOOT_SOURCE_DATA.slotItemList.unitGUIDs then
			for unitGUID, state in pairs(LOOT_SOURCE_DATA.slotItemList.unitGUIDs) do
				LOOT_SOURCE_DATA.slotItemListSave.guideList[unitGUID] = state
			end
		end
		LOOT_SOURCE_DATA.slotItemListSave[#LOOT_SOURCE_DATA.slotItemListSave+1] = LOOT_SOURCE_DATA.slotItemList
		LOOT_SOURCE_DATA.slotItemList = {}
		ClearLootSourceSaver()
	end
	LastTargetSkip = nil
end

function EventFunctions.LOOT_SLOT_CLEARED(lootSlot)
	-- that means a specific item in the list got looted
	-- also fires with fastloot
	MoveLootSlotToLootedList(lootSlot)
end

function EventFunctions.ITEM_PUSH(bagSlot, iconFileID, itemID)
	LootDataClear_NewLoot()
	itemID = iconFileID and itemIconCache[iconFileID] or itemID
	if not itemID then return end
	for slot, slotData in pairs(LOOT_SOURCE_DATA.slotItemList) do
		if slotData and slotData.itemID == itemID then
			MoveLootSlotToLootedList(slot)
		end
	end
	-- this is a second check for an item maybe it was seen in one of the last loot events and
	-- the servers have a big delay
	for i = #LOOT_SOURCE_DATA.slotItemListSave, 1, -1 do
		for slot, slotData in pairs(LOOT_SOURCE_DATA.slotItemListSave[i]) do
			if slotData and slotData.itemID == itemID then
				MoveLootSlotToLootedList(slot, LOOT_SOURCE_DATA.slotItemListSave[i])
			end
		end
	end
end

-- ###########################
-- Set the loot flags
-- Detec some extra things
-- ###########
-- Crafting
-- Gathering
-- PickPocket
-- Chests
-- Scrapping
-- ###########################

-- Scrapping spell ID
local SCRAPPING_SPELL = (C_ScrappingMachineUI and C_ScrappingMachineUI.GetScrapSpellID()) or 265742

-- Tracked Spells
local SPELL_LIST_RAW = {
    -- Professsions
    [13262] = { SUB_SOURCE_FLAG.DISENCHANTING, 	    MAIN_SOURCE_FLAG.ITEM, 	CATEGORY_FLAG.PROFESSION },
    [49383] = { SUB_SOURCE_FLAG.ENGINEERING, 		MAIN_SOURCE_FLAG.NPC, 	CATEGORY_FLAG.PROFESSION },
    [51005] = { SUB_SOURCE_FLAG.MILLING, 			MAIN_SOURCE_FLAG.ITEM, 	CATEGORY_FLAG.PROFESSION },
    [31252] = { SUB_SOURCE_FLAG.PROSPECTING, 		MAIN_SOURCE_FLAG.ITEM, 	CATEGORY_FLAG.PROFESSION },

    -- Farm professions
    [131476] = { SUB_SOURCE_FLAG.FISHING, 	MAIN_SOURCE_FLAG.ZONE, 	    CATEGORY_FLAG.PROFESSION + CATEGORY_FLAG.FARM_PROFESSION },
    [2366] = { SUB_SOURCE_FLAG.HERBALISM, 	MAIN_SOURCE_FLAG.OBJECT, 	CATEGORY_FLAG.PROFESSION + CATEGORY_FLAG.FARM_PROFESSION, "mouseover" },
    [2575] = { SUB_SOURCE_FLAG.MINING, 		MAIN_SOURCE_FLAG.OBJECT, 	CATEGORY_FLAG.PROFESSION + CATEGORY_FLAG.FARM_PROFESSION, "mouseover" },
    [8613] = { SUB_SOURCE_FLAG.SKINNING, 	MAIN_SOURCE_FLAG.NPC, 		CATEGORY_FLAG.PROFESSION + CATEGORY_FLAG.FARM_PROFESSION, "mouseover" },

    -- Classes
    [921] = { SUB_SOURCE_FLAG.PICK_POCKET, 	MAIN_SOURCE_FLAG.NPC, 	    CATEGORY_FLAG.CLASS },

    -- Looting
	[3365] = { SUB_SOURCE_FLAG.OPENING,     MAIN_SOURCE_FLAG.OBJECT,    CATEGORY_FLAG.CHEST },
	[1804] = { SUB_SOURCE_FLAG.OPENING,     MAIN_SOURCE_FLAG.OBJECT,    CATEGORY_FLAG.CHEST }, -- lockpick chest :D
	[280493] = { SUB_SOURCE_FLAG.OPENING,     MAIN_SOURCE_FLAG.OBJECT,    CATEGORY_FLAG.CHEST }, -- Scroll of Unlocking

    -- Scrapping
    [SCRAPPING_SPELL] = { SUB_SOURCE_FLAG.SCRAPPING, 	MAIN_SOURCE_FLAG.ITEM, 	        CATEGORY_FLAG.SCRAPPING },

	-- digging (DF)
	[370349] = { SUB_SOURCE_FLAG.OPENING, 	MAIN_SOURCE_FLAG.OBJECT, 	CATEGORY_FLAG.CHEST },
}

-- Formated spell list for use
-- { searchPattern, lootType, sourceFlag, categoryFlag }
local SPELL_LIST = {}
for k,v in pairs(SPELL_LIST_RAW) do
	SPELL_LIST[#SPELL_LIST+1] = {
		"^"..(GetSpellInfo(k) or k),
		v[1],
		v[2],
		v[3],
		v[4],
	}
end

local SpellNameSaver = {}
-- Check the SPELL_LIST for a spell by name
-- @return  mainSourceFlag, subSourceFlag, categoryFlag
local function CheckForSpell(spellId)
	if SpellNameSaver[spellId] then
		return SpellNameSaver[spellId][3], SpellNameSaver[spellId][2], SpellNameSaver[spellId][4], SpellNameSaver[spellId][5]
	else
		local name = GetSpellInfo(spellId)
		for i = 1, #SPELL_LIST do
			if str_match(name, SPELL_LIST[i][1]) then
				--print("CheckForSpell", SPELL_LIST[i][1])
				SpellNameSaver[spellId] = SPELL_LIST[i]
				return CheckForSpell(spellId)
			end
		end
	end
end

--local LastSpellCastStart = {}
function EventFunctions.UNIT_SPELLCAST_START(unit, spellCastGuid, spellId)
	if unit ~= "player" then return end
	--print("UNIT_SPELLCAST_START", unit, spellCastGuid, spellId)
	local _, _, _, unitId = CheckForSpell(spellId)
	if unitId then
		--print("found", GetSpellInfo(spellId), UnitGUID(unitId))
		--LastSpellCastStart[spellId] = UnitGUID(unitId)
	elseif LOOT_SPECIAL_CAST_ITEMS[spellId] then
		--LastSpellCastStart[spellId] = PlayerGUID
	end
end

local LastSpecialSpell
-- Set flags that are created with UNIT_SPELLCAST_SUCCEEDED from SPELL_LIST
function EventFunctions.UNIT_SPELLCAST_SUCCEEDED(unit, spellCast, spellId)
	if unit ~= "player" then return end
	local mainSourceFlag, subSourceFlag, categoryFlag = CheckForSpell(spellId)

	if mainSourceFlag then
		--print(LastSpellCastStart[spellId])
		----print("UNIT_SPELLCAST_SUCCEEDED", GetTimeNow(), LOOT_SOURCE_DATA.time)
		LOOT_SOURCE_DATA.mainSource = mainSourceFlag
		LOOT_SOURCE_DATA.subSourceFlag = subSourceFlag
        LOOT_SOURCE_DATA.categoryFlag = categoryFlag
		LOOT_SOURCE_DATA.time = GetTimeNow()
		LOOT_SOURCE_DATA.itemID = nil
		LastTargetSkip = true
	elseif LOOT_SPECIAL_CAST_ITEMS[spellId] then
		local src = LOOT_SPECIAL_CAST_ITEMS[spellId].flags
		LOOT_SOURCE_DATA.mainSource = src[2]
		LOOT_SOURCE_DATA.subSourceFlag = src[3]
        LOOT_SOURCE_DATA.categoryFlag = src[4]
		LOOT_SOURCE_DATA.time = GetTimeNow()
		LOOT_SOURCE_DATA.itemID = src[1]
		LastTargetSkip = true
		LastSpecialSpell = LOOT_SPECIAL_CAST_ITEMS[spellId]
	end
end
-- flags = {GGL.MAIN_SOURCE_FLAG.ITEM, GGL.SUB_SOURCE_FLAG.SPECIAL, GGL.CATEGORY_FLAG.SPECIAL},

-- ###########################
-- Loot Handle
-- ###########################
local LOOT_ITEM_PUSHED_SELF = LOOT_ITEM_PUSHED_SELF:gsub("%%s", "(.+)");
local LOOT_ITEM_PUSHED_SELF_MULTIPLE = LOOT_ITEM_PUSHED_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)")
local LOOT_ITEM_SELF = LOOT_ITEM_SELF:gsub("%%s", "(.+)")
local LOOT_ITEM_SELF_MULTIPLE = LOOT_ITEM_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)")

-- the loot!
function EventFunctions.CHAT_MSG_LOOT(msg, arg2, arg3, arg4, msgLootName, arg6, arg7, arg8, arg9, arg10, arg11, unitGUID)
	if (GGL.IS_RETAIL and unitGUID ~= PlayerGUID) or (not GGL.IS_RETAIL and msgLootName ~= PlayerName) then return end
	if ( LOOT_SOURCE_DATA.subSourceFlag == SUB_SOURCE_FLAG.SCRAPPING and LOOT_SOURCE_DATA.time and not IsValidInterval(LOOT_SOURCE_DATA.time or 0, GetTimeNow(), 500) ) then
		GGL:AddIntoErrorLog("CHAT_MSG_LOOT / source missing", msg)
		return
    end

	-- find loot
	local found, _, link, qty = str_find(msg, LOOT_ITEM_SELF_MULTIPLE)
    if not found then
        qty, found, _, link = 1, str_find(msg, LOOT_ITEM_SELF)
    end
	if not found then
        found, _, link, qty = str_find(msg, LOOT_ITEM_PUSHED_SELF_MULTIPLE)
    end
	if not found then
        qty, found, _, link = 1, str_find(msg, LOOT_ITEM_PUSHED_SELF)
	end
	if not link then return GGL:AddIntoErrorLog("CHAT_MSG_LOOT / link", msg) end
    local itemID = GetItemInfoInstant(link)
	qty = tonumber(qty)

	if LastSpecialSpell then
		if LastSpecialSpell[itemID] then
			local data = LastSpecialSpell.flags
			MoveSpecialLootSlotToLootedList("loot-special-spell", itemID, qty, data[1], data[2], data[3], data[4])
		end
		LastSpecialSpell = nil
	end

    -- push the item
	EventFunctions.ITEM_PUSH(nil, nil, itemID)
	if itemID and LOOT_SOURCE_DATA.lootedList[itemID] then
		PushItemLootFromLootedList(itemID, qty, GetItemIDData(link))
	end
end

function EventFunctions.CHAT_MSG_MONEY(msg)
	-- check for cached money entrys
	EventFunctions.ITEM_PUSH(0, 0)
	PushMoneyLootFromLootedList(ParseMoneyCopper(msg))
end

function EventFunctions.PLAYER_TARGET_CHANGED()
	LastUnitTarget = UnitGUID("target")
end

-- ###########################
-- BagItems
-- ###########################
-- some boxes are "different"
function EventFunctions.SHOW_LOOT_TOAST(typeIdentifier, itemLink, quantity, specID, sex, personalLootToast, toastMethod, lessAwesome, upgraded, corrupted)
	if not BagLootIsOpen then return end
	if LOOT_SOURCE_DATA.mainSource ~= MAIN_SOURCE_FLAG.ITEM or LOOT_SOURCE_DATA.subSourceFlag ~= SUB_SOURCE_FLAG.OPENING then return end
	if typeIdentifier == "item" then
		local itemID = GetItemInfoInstant(itemLink)
		MoveSpecialLootSlotToLootedList("item-toast", itemID, quantity, LOOT_SOURCE_DATA.itemID, MAIN_SOURCE_FLAG.ITEM, SUB_SOURCE_FLAG.OPENING, CATEGORY_FLAG.INVENTORY_CONTAINER)
	elseif typeIdentifier == "money" then

	elseif typeIdentifier == "currency" then

	end
end

-- Search bag for a itemID
local function GetItemBagAndSlot(itemLink)
	local itemID = GetItemInfoInstant(itemLink)
	for bag = 0, NUM_BAG_FRAMES do
		for slot = 1, GetContainerNumSlots(bag) do
			if GetContainerItemID(bag, slot) == itemID then
				return bag, slot, itemID
			end
		end
	end
end

-- Triggers on right click of an item
local function Hook_GetContainerItemLink(itemLink, bag, slot)
	if not itemLink then return end

	if not bag or slot then
		bag, slot = GetItemBagAndSlot(itemLink)
	end

	-- double check
	if bag and slot then
		local data, _, _, _, _, lootable, _, _, _, itemID = GetContainerItemInfo(bag, slot)
		if data and type(data) == "table" then
			lootable = data.hasLoot
			itemID = data.itemID
		end

		----print(lootable, itemID)
		if lootable then
			GGL_Parse_ToolTip:ClearLines()
			GGL_Parse_ToolTip:SetBagItem(bag, slot)
			-- after lockpick the tooltip update takes some time so check also for the last subSource from casts
			if ParseToolTipForText(GGL_Parse_ToolTip, ITEM_OPENABLE) or LOOT_SOURCE_DATA.subSourceFlag == SUB_SOURCE_FLAG.OPENING then
				ClearLootSourceSaver()
                LOOT_SOURCE_DATA.itemID = itemID
                LOOT_SOURCE_DATA.mainSource = MAIN_SOURCE_FLAG.ITEM
                LOOT_SOURCE_DATA.subSourceFlag = SUB_SOURCE_FLAG.OPENING
				LOOT_SOURCE_DATA.categoryFlag = CATEGORY_FLAG.INVENTORY_CONTAINER
				LOOT_SOURCE_DATA.time = GetTimeNow()
				BagLootIsOpen = true
			end
		end
	end
end

-- ###########################
-- Islands
-- ###########################
local IsIslandReward = false
function EventFunctions.ISLAND_COMPLETED()
	IsIslandReward = true
end

function EventFunctions.GROUP_LEFT()
	IsIslandReward = false
end

function EventFunctions.LFG_COMPLETION_REWARD()
	if not IsIslandReward then return end
	local name, typeID, subtypeID, iconTextureFile, moneyBase, moneyVar, experienceBase, experienceVar, numStrangers, numRewards = GetLFGCompletionReward()
	if not numRewards or numRewards == 0 then
		return
	end

	ClearLootSourceSaver()
	LOOT_SOURCE_DATA.mainSource = MAIN_SOURCE_FLAG.ZONE
	LOOT_SOURCE_DATA.subSourceFlag = SUB_SOURCE_FLAG.ISLAND_EXPEDITION
	LOOT_SOURCE_DATA.categoryFlag = CATEGORY_FLAG.LOOT
	LOOT_SOURCE_DATA.time = GetTimeNow()

	for i = 1, numRewards do
		local texture, quantity, isBonus, bonusQuantity, name, quality, itemID, objectType = GetLFGCompletionRewardItem(i);
		if objectType == "item" then
			MoveSpecialLootSlotToLootedList("item-lfg-complete", itemID, quantity, nil, MAIN_SOURCE_FLAG.ZONE, SUB_SOURCE_FLAG.ISLAND_EXPEDITION, CATEGORY_FLAG.LOOT)
		end
	end
end

-- ###########################
-- Quests
-- ###########################
function EventFunctions.QUEST_LOOT_RECEIVED(questID, itemLink, quantity)
	local subSource = IsWorldQuest(questID) and SUB_SOURCE_FLAG.WORLD_QUEST or SUB_SOURCE_FLAG.QUEST
	local itemID = GetItemInfoInstant(itemLink)
	MoveSpecialLootSlotToLootedList("quest-loot", itemID, quantity, questID, MAIN_SOURCE_FLAG.ZONE, subSource, CATEGORY_FLAG.QUEST)
end

function EventFunctions.QUEST_TURNED_IN(questID, xpReward, moneyReward)
	local subSource = IsWorldQuest(questID) and SUB_SOURCE_FLAG.WORLD_QUEST or SUB_SOURCE_FLAG.QUEST

	PushMoneyLootDirect(moneyReward, questID, MAIN_SOURCE_FLAG.ZONE, subSource, CATEGORY_FLAG.QUEST)
end

-- ###########################
-- Mission table
-- ###########################
local lastMissionCompleted
local missionTableRewardList

function EventFunctions.GARRISON_MISSION_BONUS_ROLL_COMPLETE(missionID, success)
	if success then
		lastMissionCompleted = missionID
	else
		lastMissionCompleted = nil
	end
end

function EventFunctions.GARRISON_MISSION_BONUS_ROLL_LOOT(itemID, quantity)
	if not lastMissionCompleted then return end
	if itemID then
		if itemID == 0 then
			PushMoneyLootDirect(quantity, lastMissionCompleted, MAIN_SOURCE_FLAG.EXTRA, SUB_SOURCE_FLAG.MISSION_TABLE, CATEGORY_FLAG.MISSION_TABLE)
		else
			-- moved to GARRISON_MISSION_COMPLETE_RESPONSE
		end
	end
end

function EventFunctions.GARRISON_MISSION_COMPLETE_RESPONSE(missionID, canComplete, success, overmaxSucceeded, followerDeaths, autoCombatResult)
	if not missionTableRewardList or not missionTableRewardList[missionID] then return end
	for itemID, itemQuantity in pairs(missionTableRewardList[missionID]) do
		MoveSpecialLootSlotToLootedList("mission-loot", itemID, itemQuantity, missionID, MAIN_SOURCE_FLAG.EXTRA, SUB_SOURCE_FLAG.MISSION_TABLE, CATEGORY_FLAG.MISSION_TABLE)
	end
end

function EventFunctions.GARRISON_MISSION_NPC_OPENED(followerTypeID)
	missionTableRewardList = {}
	if not followerTypeID then return end
	local missionsList = C_Garrison.GetCompleteMissions(followerTypeID)
	if not missionsList or #missionsList < 1 then return end

	for missionIndex, missionData in ipairs(missionsList) do
		if missionData.rewards then
			for rewardIndex, rewardData in ipairs(missionData.rewards) do
				if rewardData.itemID then
					missionTableRewardList[missionData.missionID] = missionTableRewardList[missionData.missionID] or {}
					missionTableRewardList[missionData.missionID][rewardData.itemID] = rewardData.quantity
				end
			end
		 end
	end
end

-- ###########################
-- Init
-- ###########################
local function UseContainerItem_Hook(bag, slot, target)
	if not target then
		Hook_GetContainerItemLink(GetContainerItemLink(bag, slot), bag, slot)
	end
end


local function Init()

	hooksecurefunc("UseItemByName", function(name, target)
        if not target then
            Hook_GetContainerItemLink(select(2, GetItemInfo(name)))
        end
    end)

	if UseContainerItem then
		hooksecurefunc("UseContainerItem", UseContainerItem_Hook)
	end
	if C_Container and C_Container.UseContainerItem then
		hooksecurefunc(C_Container, "UseContainerItem", UseContainerItem_Hook)
	end

	PlayerGUID = UnitGUID("player")
	PlayerName = UnitName("player")
end
GGL:AddInitFuncPlayerLogin(Init)
