local AddonName, GGL = ...
local GUI = GGL.GUI
local Global = {}
GGL.Options.LootAlert = Global

local L = GGL.Locale

-- lua
local tbl_remove = table.remove

-- WoW

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- ##################
-- local
-- ##################
local GUISave = {}
local IsGlobalDb

local DbTable
-- ##################
-- local helper
-- ##################
local function SetLinePositionsFromList()
    local children = GUISave.list.children
    for i = 1, #children do
        children[i]:SetPosition(i, #children)
    end
end

local function RemoveEntryFromListAndDB(ruleLine)
    if not ruleLine then return end

    local db = DbTable

    -- check if ruleLine is the same as db entry
    if ruleLine.dbRule ~= db[ruleLine.position] then return end

    -- first remove from list
    local posFix = 0
    local children = GUISave.list.children
    local numChildern = #children
    for i = 1, numChildern do
        -- save child
        local child = children[i]
        children[i] = nil

        -- check if the child is the one that gets removed
        if child == ruleLine then
            child:Release()
            posFix = posFix + 1
        else
            children[i-posFix] = child
        end
    end
    GUISave.list:DoLayout()

    -- remove from db
    tbl_remove(db, ruleLine.position)

    -- reset line positions
    SetLinePositionsFromList()
end

local function SwapListPosition(srcPos, destPos)
    if not srcPos or not destPos then return end

    -- get childrenlist and db
    local children = GUISave.list.children
    local db = DbTable

    -- get childs
    local srcChild = children[srcPos]
    local destChild = children[destPos]

    -- swap in list
    children[destPos] = srcChild
    children[srcPos] = destChild
    GUISave.list:DoLayout()

    -- swap db
    db[destPos] = children[destPos].dbRule
    db[srcPos] = children[srcPos].dbRule

    -- reset line positions
    SetLinePositionsFromList()
end

-- ##################
-- options
-- ##################
local function WidgetOnRelease(self)
    wipe(GUISave)
    DbTable = nil
    IsGlobalDb = nil
end

local function Enabled_OnValueChanged(self, event, state)
    if IsGlobalDb then
        GGL.db.GlobalAlertRules.disabled = not state
    else
        GGL.db.AlertRules.disabled = not state
    end
end

local function DisableInSession_OnValueChanged(self, event, state)
    GGL.db.GlobalAlertRules.disabledInChallenge = state
end

local function Line_DeleteOnClick(self, event, mb)
    GGL.Options:OpenConfirm(
        format(L["Delete rule '|cffff0000%s|r'?"], self.dbRule.name),
        RemoveEntryFromListAndDB,
        nil,
        self
    )
end

local function Line_MoveOnClick(self, event, mb)
    if event == "UpOnClick" then
        SwapListPosition(self.position, self.position-1)
    elseif event == "DownOnClick" then
        SwapListPosition(self.position, self.position+1)
    end
end

local function AddRule_OnClick(self, event, mb)
    -- create new rule table
    local rule = {
        name = L["New Rule"]
    }

    -- add into db
    DbTable[#DbTable+1] = rule

    -- add a new ruleLine
    local ruleLine = AceGUI:Create("GoldGoblinLootGUIWidget-LootAlertLine")
    ruleLine:SetFullWidth(true)
    ruleLine:SetRule(rule)
    ruleLine:SetCallback("DeleteOnClick", Line_DeleteOnClick)
    ruleLine:SetCallback("UpOnClick", Line_MoveOnClick)
    ruleLine:SetCallback("DownOnClick", Line_MoveOnClick)
    ruleLine:SetGlobal(IsGlobalDb)
    GUISave.list:AddChild(ruleLine)

    -- set new line positions
    SetLinePositionsFromList()

    -- open edit frame
    ruleLine:ShowEditFrame()
end

local function Options(AceGUI, container)
    DbTable = DbTable or GGL.db.AlertRules.Rules

    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    container:AddChild(scroll)

    local top = AceGUI:Create("SimpleGroup")
    top:SetFullWidth(true)
    top:SetLayout("Flow")
    scroll:AddChild(top)

    local description = AceGUI:Create("Label")
    description:SetFullWidth(true)
    description:SetFontObject(GameFontHighlight)
    description:SetJustifyH("CENTER")
    description:SetJustifyV("MIDDLE")
    if IsGlobalDb then
        description:SetText(L["These rules don't require an active session or challenge."])
    else
        description:SetText(L["These rules are only active in challenges."])
    end
    top:AddChild(description)

    local line = AceGUI:Create("Heading")
    --line:SetText(L["Rules"])
    line:SetFullWidth(true)
    top:AddChild(line)

    local addRule = AceGUI:Create("Button")
    addRule:SetText(L["Add a new alert rule."])
    addRule:SetAutoWidth(true)
    addRule:SetCallback("OnClick", AddRule_OnClick)
    top:AddChild(addRule)

    local enabled = AceGUI:Create("CheckBox")
    enabled:SetLabel(ENABLE)
    enabled:SetWidth(100)
    if IsGlobalDb then
        enabled:SetValue(not GGL.db.GlobalAlertRules.disabled)
    else
        enabled:SetValue(not GGL.db.AlertRules.disabled)
    end
    enabled:SetCallback("OnValueChanged", Enabled_OnValueChanged)
    top:AddChild(enabled)

    if IsGlobalDb then
        local disableInSession = AceGUI:Create("CheckBox")
        disableInSession:SetLabel(L["Disable while in challenge"])
        disableInSession:SetValue(GGL.db.GlobalAlertRules.disabledInChallenge)
        disableInSession:SetCallback("OnValueChanged", DisableInSession_OnValueChanged)
        top:AddChild(disableInSession)
    end

    local line1 = AceGUI:Create("Heading")
    line1:SetFullWidth(true)
    top:AddChild(line1)

    local list = AceGUI:Create("ScrollFrame")
    list:SetFullWidth(true)
    list:SetLayout("List")
    list:SetHeight(container.content:GetHeight() - top.frame:GetHeight() - 10)
    GUISave.list = list
    scroll:AddChild(list)

    for ruleID, rule in ipairs(DbTable) do
        local ruleLine = AceGUI:Create("GoldGoblinLootGUIWidget-LootAlertLine")
        ruleLine:SetFullWidth(true)
        ruleLine:SetRule(rule)
        ruleLine:SetPosition(ruleID, #DbTable)
        ruleLine:SetCallback("DeleteOnClick", Line_DeleteOnClick)
        ruleLine:SetCallback("UpOnClick", Line_MoveOnClick)
        ruleLine:SetCallback("DownOnClick", Line_MoveOnClick)
        ruleLine:SetGlobal(IsGlobalDb)
        list:AddChild(ruleLine)
    end

    -- resize scrollFrame
    scroll:DoLayout()
    list:DoLayout()
end

local function Options_GlobalAlerts(AceGUI, container)
    DbTable = GGL.db.GlobalAlertRules.Rules
    IsGlobalDb = true

    Options(AceGUI, container)
end

-- Add into Options
GGL.Options:AddDisabledIntoTree(nil, L["User Settings"], "userSettings")
GGL.Options:AddIntoTree(Options_GlobalAlerts, L["Alerts: Global"], "lootalert_global")
GGL.Options:AddIntoTree(Options, L["Alerts: Challenge"], "lootalert")