local AddonName, GGL = ...
local GUI = GGL.GUI
local Templates = GGL.Templates
local Global = {}
GGL.Options.Multibox = Global

local L = GGL.Locale

-- lua
local pairs, ipairs = pairs, ipairs
local wipe = table.wipe
local match, format = string.match, string.format

-- WoW
local UnitFactionGroup = UnitFactionGroup
local IsShiftKeyDown = IsShiftKeyDown

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- GGL
local GetPlayerFromSync = Templates.Sync.GetPlayerFromSync

-- ##################
-- local
-- ##################

-- ##################
-- local helper
-- ##################

-- ##################
-- options
-- ##################

local function ItemFilterDropDown_OnValueChanged(self, event, value)
    GGL.db.MultiboxOptions.secondItemFilter = value or "Base"
end

local function GetSetDropDownList(setType)
    local db = GGL.db.Config.Sets[setType]

    local list = {}
    for k,v in pairs(db) do
        list[k] = v.name
    end

    return list
end

local function AutoAcceptKey_OnEnterPressed(self, event, text)
    GGL.db.MultiboxOptions.autoAcceptKey = text == "" and nil or text

    local sessionOption = self:GetUserData("sessionOption")
    sessionOption:SetDescription(GGL.db.MultiboxOptions.autoAcceptKey)

    self:ClearFocus()
end

local function AddEmaTeam_OnClick(self, button)
    local emaTeamRaw = _G.EMAApi.TeamList()
    local emaTeam = {}
    for name, sortOrder in _G.EMAApi.TeamList() do
        emaTeam[sortOrder] = name
    end
    if #emaTeam < 0 then return end

    local sessionOption = self:GetUserData("sessionOption")
    sessionOption:RemoveAllPlayer()

    for index, name in ipairs(emaTeam) do
        sessionOption:AddPlayer(Templates:Get("WoWPlayer", name))
    end
end

local function AddISBoxerTeam_OnClick(self, button)
    local team = _G.isboxer.CharacterSet.Members
    if team and #team < 1 then return end

    local sessionOption = self:GetUserData("sessionOption")
    sessionOption:RemoveAllPlayer()

    for index, name in ipairs(team) do
        sessionOption:AddPlayer(Templates:Get("WoWPlayer", name))
    end
end

-- ov
local function OV_ReloadSettings(self)
    if not self.inviteTemplate or not self.session:IsRunningTemplate() then return end

    self.session:SetData(self.db.mainSet or "Base", self.db.secondItemFilter or "Base", true)
    self.session:ReloadSet()
end

local function OV_StartSession(self)
    if not self.session:DataIsSet() then
        self.session:SetData(self.db.duration*60, self.db.mainSet or "Base")
    end
    self:RawCall("StartSession")
end

local function Options(AceGUI, container)
    local db = GGL.db.MultiboxOptions

    local sessionOption = GGL.Session:Options_Add(container, "M", GGL.db.MultiboxOptions)

    if not sessionOption then return end

    sessionOption:GUI_AddInviteByName()
    sessionOption:OverrideFunction("ReloadSettings", OV_ReloadSettings)
    sessionOption:OverrideFunction("StartSession", OV_StartSession)


    local rightItemFilterDropdown = AceGUI:Create("Dropdown")
    rightItemFilterDropdown:SetList(GetSetDropDownList(GGL.SET_ID_LIST.Item))
    rightItemFilterDropdown:SetValue(db.secondItemFilter or "Base")
    rightItemFilterDropdown:SetLabel(L["Right Item Filter"])
    rightItemFilterDropdown:SetCallback("OnValueChanged", ItemFilterDropDown_OnValueChanged)
    sessionOption.gui.topGroup:AddChild(rightItemFilterDropdown)
    sessionOption.gui.rightItemFilterDropdown = rightItemFilterDropdown


    local autoAcceptKey = AceGUI:Create("EditBox")
    autoAcceptKey:SetWidth(200)
    autoAcceptKey:SetLabel(L["Auto Accept key"])
    autoAcceptKey:SetCallback("OnEnterPressed", AutoAcceptKey_OnEnterPressed)
    autoAcceptKey:SetText(GGL.db.MultiboxOptions.autoAcceptKey)
    autoAcceptKey:SetUserData("sessionOption", sessionOption)
    sessionOption.gui.topGroup:AddChild(autoAcceptKey)
    sessionOption.gui.autoAcceptKey = autoAcceptKey

    if _G.isboxer and #_G.isboxer.CharacterSet.Members > 0 then
        local inviteISBoxerTeam = AceGUI:Create("Button")
        inviteISBoxerTeam:SetText(format(L["Add '%s' Team"], "ISBoxer"))
        inviteISBoxerTeam:SetAutoWidth(true)
        inviteISBoxerTeam:SetCallback("OnClick", AddISBoxerTeam_OnClick)
        inviteISBoxerTeam:SetUserData("sessionOption", sessionOption)
        sessionOption.gui.bottomExtraGroup:AddChild(inviteISBoxerTeam)
    end
    --EMAApi.TeamList() - [name] - order
    if _G.EMAApi then
        local inviteEMATeam = AceGUI:Create("Button")
        inviteEMATeam:SetText(format(L["Set '%s' Team"], "EMA"))
        inviteEMATeam:SetAutoWidth(true)
        inviteEMATeam:SetCallback("OnClick", AddEmaTeam_OnClick)
        inviteEMATeam:SetUserData("sessionOption", sessionOption)
        sessionOption.gui.bottomExtraGroup:AddChild(inviteEMATeam)
    end

    -- resize scrollFrame
    if sessionOption.gui.scroll then
        sessionOption.gui.scroll:DoLayout()
    end
end

-- Add into Options
GGL.Options:AddIntoTree(Options, L["Multibox"], "multibox")