local AddonName, GGL = ...

local LibStub = _G.LibStub

local Options = {}
GGL.Options = Options

GGL:AddCallbackHandler(Options)

local format = string.format

local AceGUI = LibStub("AceGUI-3.0")
local L = GGL.Locale

local OptionsCreated, BlizzardOptionsCreated = false, false

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

-- local
local FirstTreeValue
local cb_counter = 0
local TreeCallbackRegister = {}
local TreeRegister = {}
local OptionsFrame

local function GetTreeTable(gTable, tTable, c, value)
	for i = 1, #tTable do
		if tTable[i].value == gTable[c] then
			value = c == 1 and gTable[c] or value.."\001"..tTable[i].value
			if c == #gTable then
				if not tTable[i].children then tTable[i].children = {} end
				return tTable[i].children, value
			else
				return GetTreeTable(gTable, tTable[i].children, c+1, value)
			end
		end
	end
	error(gTable[c].." not found.")
end

local function TreeGroupOnSelect(self, event, group)
	OptionsFrame.treeGroup:ReleaseChildren()
	TreeCallbackRegister[group](AceGUI, self)
end

local FRAME_WIDTH, FRAME_HEIGHT = 780, 650
local FRAME_WIDTH_RESIZE, FRAME_HEIGHT_RESIZE = 500, 500

local function RefreshHeader()
	local db = GGL.db.Settings.Streamer
	local frame = OptionsFrame.topFrame

	if not db.enabled then
		frame.topFrameTexture:Show()
		if frame.title then
			frame.title:Hide()
			frame.battleTag:Hide()
		end
	else
		if not frame.title then
			local title = frame:CreateFontString(nil, "BACKGROUND", "GameFontNormalLarge")
			title:SetAllPoints(frame.addonVersion)
			title:SetJustifyH("RIGHT")
			title:SetJustifyV("MIDDLE")
			title:SetWordWrap(false)
			frame.title = title

			local battleTag = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightLarge")
			battleTag:SetAllPoints(title)
			battleTag:SetJustifyH("RIGHT")
			battleTag:SetJustifyV("BOTTOM")
			battleTag:SetWordWrap(false)
			frame.battleTag = battleTag
		end

		--[[
		if db.hideBanner then
			frame.topFrameTexture:Hide()
			frame.title:SetJustifyH("CENTER")
		else
			frame.topFrameTexture:Show()
			frame.title:SetJustifyH("RIGHT")
		end
		]]

		frame.title:SetText(db.title)
		frame.title:Show()

		local _, battleTag = BNGetInfo()
		frame.battleTag:SetText(format(L["BattleTag: %s"], db.battleTag or battleTag))
		frame.battleTag:Show()
	end
end

local function OptionFrame_OnShow()

end

local function OptionFrame_OnClose()

end

local function OpenOptionsFrame()
	-- check if bnet is found
	-- sometimes bNet gets no connection...
	local _, btag = BNGetInfo()
	if not btag then
		GGL.Options:OpenInformation(
			L["Warning your battle.net is offline.\nRestart the game and check if your Friend List is available."]
		)
		return
	end
	if not OptionsFrame then
		local window = AceGUI:Create("Window") -- never free after creation :)
		window:SetLayout("Fill")
		window:SetCallback("OnShow", OptionFrame_OnShow)
		window:SetCallback("OnClose", OptionFrame_OnClose)
        window:SetTitle("GoldGoblinLoot - "..GGL.ADDON_VERSION)

		local frame = window.frame
		local content = window.content

		-- some frame settings
		if frame.SetResizeBounds then
			frame:SetResizeBounds(FRAME_WIDTH, FRAME_HEIGHT, FRAME_WIDTH + FRAME_WIDTH_RESIZE,FRAME_HEIGHT + FRAME_HEIGHT_RESIZE)
		else
			frame:SetMinResize(FRAME_WIDTH,FRAME_HEIGHT)
			frame:SetMaxResize(FRAME_WIDTH + FRAME_WIDTH_RESIZE,FRAME_HEIGHT + FRAME_HEIGHT_RESIZE)
		end
        frame:SetClampedToScreen(true)
        frame:SetToplevel(true)
        --window settings
        window:SetWidth(FRAME_WIDTH)
		window:SetHeight(FRAME_HEIGHT)
		-- Save the frame with a global name so it can be added into special frames
		_G["GoldGoblinLootOptionsFrame"] = frame
		tinsert(UISpecialFrames, "GoldGoblinLootOptionsFrame")	-- allow ESC close

		-- top frame
		local topFrame = GGL.GUI:CreateBorderedFrame(frame)
		topFrame:SetPoint("TOPLEFT",frame,"TOPLEFT",12,-32)
		topFrame:SetPoint("RIGHT",frame,"RIGHT",-12,0)
		topFrame:SetBackdropColor(0.05, 0.05, 0.05, 0.6)
		topFrame:SetHeight(78)
		window.topFrame = topFrame

		-- top frame - texture
		local topFrameTexture = topFrame:CreateTexture(nil, "ARTWORK")
		topFrameTexture:SetPoint("LEFT", topFrame, "LEFT", 0, 0)
		topFrameTexture:SetSize(768, 72)
		topFrameTexture:SetTexture(GGL.BANNER_IMAGE)
		topFrameTexture:SetTexCoord(unpack(GGL.BANNER_IMAGE_TEX["TOP"]))
		topFrame.topFrameTexture = topFrameTexture

		-- easter egg
		local eggFrame = CreateFrame("Button", nil, topFrame)
		eggFrame:SetSize(20,20)
		eggFrame:SetPoint("CENTER", topFrameTexture, "TOPLEFT", 98, -25)
		eggFrame.clickCount = 0
		eggFrame:SetScript("OnClick", function(self, mb)
			if not self.lastClick or ((time() - self.lastClick) > 2) then
				self.clickCount = 1
				self.lastClick = time()
			else
				self.clickCount = self.clickCount + 1
			end
			if self.clickCount >= 5 then
				PlaySoundFile(3169894, "master")
				self.lastClick = nil
			end
		end)

		-- top frame - addon version
		local addonVersion = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
		addonVersion:SetPoint("TOPLEFT", topFrame, "TOPLEFT", 5, -5)
		addonVersion:SetPoint("BOTTOMRIGHT", topFrame, "BOTTOMRIGHT", -5, 5)
		addonVersion:SetJustifyH("RIGHT")
		addonVersion:SetJustifyV("TOP")
		addonVersion:SetWordWrap(false)
		addonVersion:SetText(GGL.ADDON_VERSION)
		topFrame.addonVersion = addonVersion

		-- bottom frame - texture
		local bottomFrameTexture = frame:CreateTexture(nil, "ARTWORK")
		bottomFrameTexture:SetPoint("BOTTOM", content, "BOTTOM", 0, -25)
		bottomFrameTexture:SetSize(768, 32)
		bottomFrameTexture:SetTexture(GGL.BANNER_IMAGE)
		bottomFrameTexture:SetTexCoord(unpack(GGL.BANNER_IMAGE_TEX["BOTTOM"]))

		-- change position of the content
		content:ClearAllPoints()
		content:SetPoint("TOPLEFT", topFrame, "BOTTOMLEFT", 0, -2)
		content:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-12,32) -- 13

		local treeGroup = AceGUI:Create("TreeGroup")
		treeGroup:SetTreeWidth(200, false)
		treeGroup:SetTree(TreeRegister)
		treeGroup:SetLayout("Fill")
		treeGroup:SetCallback("OnGroupSelected", TreeGroupOnSelect)
		window:AddChild(treeGroup)
		window.treeGroup = treeGroup

		OptionsFrame = window

		treeGroup:SetSelected(FirstTreeValue)

		RefreshHeader()

		Options:RegisterCallbackEvent("DbOnSet-Settings-Streamer-enabled", RefreshHeader)
		Options:RegisterCallbackEvent("DbOnSet-Settings-Streamer-hideBanner", RefreshHeader)
		Options:RegisterCallbackEvent("DbOnSet-Settings-Streamer-title", RefreshHeader)
		Options:RegisterCallbackEvent("DbOnSet-Settings-Streamer-battleTag", RefreshHeader)
	end

	OptionsFrame:Show()
end

local function CreateBlizzardOptions()
	local options = {
		type = "group",
		name = "GoldGoblinLoot",
		args = {
			openOptions = {
				order = 1,
				type = 'execute',
				name = L["Open Options"],
				func = function()
					InterfaceOptionsFrame:Hide()
					OpenOptionsFrame()
				end,
			},
		}
	}
	AceConfig:RegisterOptionsTable("GoldGoblinLoot-Options", options)
	local blizzOptions = AceConfigDialog:AddToBlizOptions("GoldGoblinLoot-Options", "GoldGoblinLoot")
end

SLASH_GGL1 = "/ggl"
SLASH_GGL2 = "/goldgoblinloot"
local blizzPanel
SlashCmdList["GGL"] = function(msg)
	if msg == "error" then
		GGL.Export:FillExportWindow(GGL:GetErrorLogAsString(), "text")
	elseif msg == "-error" then
		GGL:PurgeErrorLog()
		GGL:Print("Error log cleared.")
	elseif msg == "fix" then
		-- clear session db
		wipe(GoldGoblinLootSessionDB)
		-- clear errors
		GGL:PurgeErrorLog()
		-- reset alls base options
		if GGL.db and GGL.db.Config and GGL.db.Config.Sets then
			for key, data in pairs(GGL.db.Config.Sets) do
				if data.Base then
					data.Base = nil
				end
			end
		end
		-- reload UI
		_G.C_UI.Reload()
	elseif msg == "port" then
		GGL:PortOutOfDungeon()
	elseif msg == "mapid" then
		local zoneID, mapName, mapTypeFlag, difficultyID = GGL:GetZoneData()
		GGL:Print(format("zoneID: %d, difficultyID: %d, mapName: %s", zoneID or 0, difficultyID or 0, mapName or "nil"))
	elseif msg == "restore" then
		if GGL.db.LastChallengeExport and GGL.db.LastChallengeExport ~= "" then
			GGL.Export:FillExportWindow(GGL.db.LastChallengeExport, "csv")
		end
	else
		OpenOptionsFrame()
	end
end

local registerFrame = CreateFrame("Frame", nil, InterfaceOptionsFrame)
registerFrame:SetScript("OnShow", function(self)
	if not BlizzardOptionsCreated then
		CreateBlizzardOptions()
		BlizzardOptionsCreated = true
	end
	self:SetScript("OnShow", nil)
end)

--- Open Options frame
function Options:Open()
	OpenOptionsFrame()
end

-- Add into left options tree
-- @param 		method		<func> 		loaded into right window (AceGUI, container)
-- @param		name		<string>	name in the tree
-- @param		value		<string>	tree value
-- @param		...			<string> 	create option under other tree member
function Options:AddIntoTree(method, name, value, ...)
	local treeGroupes = { ... }
	value = value or name
	if not FirstTreeValue then FirstTreeValue = value end

	local cValue
	local t
	if #treeGroupes >= 1 then
		t, cValue = GetTreeTable(treeGroupes, TreeRegister, 1)
	else
		t = TreeRegister
	end
	t[ #t + 1] = {
		text = name,
		value = value,
	}
	cValue = cValue and cValue.."\001"..value or value
	TreeCallbackRegister[cValue] = method -- OnGroupSelected
end

-- Add a disabled line into left options tree
-- @param 		method		<func> 		loaded into right window (AceGUI, container)
-- @param		name		<string>	name in the tree
-- @param		value		<string>	tree value
-- @param		...			<string> 	create option under other tree member
function Options:AddDisabledIntoTree(method, name, value, ...)
	local treeGroupes = { ... }
	value = value or name
	if not FirstTreeValue then FirstTreeValue = value end

	local cValue
	local t
	if #treeGroupes >= 1 then
		t, cValue = GetTreeTable(treeGroupes, TreeRegister, 1)
	else
		t = TreeRegister
	end
	t[ #t + 1] = {
		text = "",
		value = value.."empty",
		disabled = true,
	}
	t[ #t + 1] = {
		text = "-- "..name,
		value = value,
		disabled = true,
	}
	cValue = cValue and cValue.."\001"..value or value
	TreeCallbackRegister[cValue] = method -- OnGroupSelected
end

-- Opens a confirmation frame
-- @param 		text		<string> 	display text
-- @param		onAccept	<func>		On accept button click
-- @param		onCancel	<func>		On cancel button click
-- @param		...			<...>		values
function Options:OpenConfirm(text, onAccept, onCancel, ...)
	local frame = self.confirmFrame
	if not frame then
		-- from AceConfigDialog
		frame = CreateFrame("Frame", nil, UIParent)
		self.confirmFrame = frame
		frame:Hide()
		frame:SetPoint("CENTER", UIParent, "CENTER")
		frame:SetSize(320, 72)
		frame:SetFrameStrata("TOOLTIP")
		frame:SetScript("OnKeyDown", function(self, key)
			if key == "ESCAPE" then
				self:SetPropagateKeyboardInput(false)
				if self.cancel:IsShown() then
					self.cancel:Click()
				else -- Showing a validation error
					self:Hide()
				end
			else
				self:SetPropagateKeyboardInput(true)
			end
		end)

		local text = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
		text:SetSize(290, 0)
		text:SetPoint("TOP", 0, -16)
		frame.text = text

		local border = CreateFrame("Frame", nil, frame, "DialogBorderDarkTemplate")
		border:SetAllPoints(frame)

		local function newButton(text)
			local button = CreateFrame("Button", nil, frame)
			button:SetSize(128, 21)
			button:SetNormalFontObject(GameFontNormal)
			button:SetHighlightFontObject(GameFontHighlight)
			button:SetNormalTexture(130763) -- "Interface\\Buttons\\UI-DialogBox-Button-Up"
			button:GetNormalTexture():SetTexCoord(0.0, 1.0, 0.0, 0.71875)
			button:SetPushedTexture(130761) -- "Interface\\Buttons\\UI-DialogBox-Button-Down"
			button:GetPushedTexture():SetTexCoord(0.0, 1.0, 0.0, 0.71875)
			button:SetHighlightTexture(130762) -- "Interface\\Buttons\\UI-DialogBox-Button-Highlight"
			button:GetHighlightTexture():SetTexCoord(0.0, 1.0, 0.0, 0.71875)
			button:SetText(text)
			return button
		end

		local accept = newButton(ACCEPT)
		accept:SetPoint("BOTTOMRIGHT", frame, "BOTTOM", -6, 16)
		accept:SetScript("OnClick", function(self)
			frame:Hide()
			if self.func then
				self.func(unpack(self.obj.data))
			end
		end)
		accept.obj = frame
		frame.accept = accept

		local cancel = newButton(CANCEL)
		cancel:SetPoint("LEFT", accept, "RIGHT", 13, 0)
		cancel:SetScript("OnClick", function(self)
			frame:Hide()
			if self.func then
				self.func(unpack(self.obj.data))
			end
		end)
		cancel.obj = frame
		frame.cancel = cancel
	end

	frame.accept.func = onAccept
	frame.cancel.func = onCancel
	frame.data = {...}

	frame.text:SetText(text)

	frame:SetHeight(frame.text:GetStringHeight() + 56)

	frame:Show()
end

-- Opens a information frame with Okay Button
-- @param 		text		<string> 	display text
-- @param		onAccept	<func>		On accept button click
-- @param		...			<...>		values
function Options:OpenInformation(text, onAccept, ...)
	local frame = self.confirmFrame
	if not frame then
		-- from AceConfigDialog
		frame = CreateFrame("Frame", nil, UIParent)
		self.confirmFrame = frame
		frame:Hide()
		frame:SetPoint("CENTER", UIParent, "CENTER")
		frame:SetSize(320, 72)
		frame:SetFrameStrata("TOOLTIP")
		frame:SetScript("OnKeyDown", function(self, key)
			if key == "ESCAPE" then
				self:SetPropagateKeyboardInput(false)
				if self.cancel:IsShown() then
					self.cancel:Click()
				else -- Showing a validation error
					self:Hide()
				end
			else
				self:SetPropagateKeyboardInput(true)
			end
		end)

		local text = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
		text:SetSize(290, 0)
		text:SetPoint("TOP", 0, -16)
		frame.text = text

		local border = CreateFrame("Frame", nil, frame, "DialogBorderDarkTemplate")
		border:SetAllPoints(frame)

		local function newButton(text)
			local button = CreateFrame("Button", nil, frame)
			button:SetSize(128, 21)
			button:SetNormalFontObject(GameFontNormal)
			button:SetHighlightFontObject(GameFontHighlight)
			button:SetNormalTexture(130763) -- "Interface\\Buttons\\UI-DialogBox-Button-Up"
			button:GetNormalTexture():SetTexCoord(0.0, 1.0, 0.0, 0.71875)
			button:SetPushedTexture(130761) -- "Interface\\Buttons\\UI-DialogBox-Button-Down"
			button:GetPushedTexture():SetTexCoord(0.0, 1.0, 0.0, 0.71875)
			button:SetHighlightTexture(130762) -- "Interface\\Buttons\\UI-DialogBox-Button-Highlight"
			button:GetHighlightTexture():SetTexCoord(0.0, 1.0, 0.0, 0.71875)
			button:SetText(text)
			return button
		end

		local accept = newButton(OKAY)
		accept:SetPoint("BOTTOM", frame, "BOTTOM", 0, 16)
		accept:SetScript("OnClick", function(self)
			frame:Hide()
			if self.func then
				self.func(unpack(self.obj.data))
			end
		end)
		accept.obj = frame
		frame.accept = accept
	end

	frame.accept.func = onAccept
	frame.data = {...}

	frame.text:SetText(text)

	frame:SetHeight(frame.text:GetStringHeight() + 56)

	frame:Show()
end

local DB_SETTER_EVENT_F = "DbOnSet-%s-%s-%s"
local DB_GETTER_SETTER_PROTO = {
	["Get"] = function(self, key)
		return self.db[key]
	end,
	["Set"] = function(self, key, value)
		self.db[key] = value
		local event = DB_SETTER_EVENT_F:format(self.category, self.subCategory, tostring(key))
		Options:FireCallbackEvent(event, event, self.db, key, value)
	end,
}
local DB_GETTER_SETTER_MT = {__index = DB_GETTER_SETTER_PROTO}

--- Creates a get and set function for options
-- Fires a cb on Options with format "DbOnSet-%s-%s-%s" ("DbOnSet-category-subCategory-key"(event, db, key, value))
-- @param		dbTable			<table>
-- @param		category		<string> dbTable[category]
-- @param		subCategory		<string> dbTable[category][subCategory]
function Options:GetDbGetterAndSetter(dbTable, category, subCategory)
	category = tostring(category)
	subCategory = tostring(subCategory)
	assert(dbTable[category], format("Category '%s' not found in dbTable", category))
	assert(dbTable[category][subCategory], format("SubCategory '%s' not found in dbTable", category))

	local t = {
		db = dbTable[category][subCategory],
		category = category,
		subCategory = subCategory,
	}
	setmetatable(t, DB_GETTER_SETTER_MT)

	return t
end